/******************************************************************************************************/
/* CALCULATE CALORIE AVAILABILITY BY HOUSEHOLD IN 2000							      */
/******************************************************************************************************/

/* use a semicolon as the command delimiter */
#delimit ;

clear;

set matsize 800;
set memory 100m;

/******************************************************************************************************/
/* Input kilo calories per 100g as conversion factors for foods derived from "Tabla de 		      */
/* Composición de Alimentos de Centroamerica" accessed online at                                     */
/* (http://www.tabladealimentos.net/tca/TablaAlimentos/inicio.html). The data appendix provides full  */
/* details and sources.                                                                               */
/******************************************************************************************************/

use "$RD00\g_grams00", clear;
save "$RD00\g_calories00", replace;

use "$BASE\calorie_content", clear;
g food = real(code_rps);
rename kcals_per_100g kcal_per_gram;
replace kcal_per_gram = 282.00 if food == 14;		/* use INEC value for huesos de res/cerdo */
replace per_edible = 1 if food == 14;			/* assume 100 percent edible for huesos */
collapse kcal_per_gram per_edible, by(food);
sort food;
save "$BASE\temp", replace;

use "$RD00\g_calories00", clear;
sort food;
merge food using "$BASE\temp";
tab _m;
bys _m: tab food, missing;
drop if _m==2;
drop _m;
erase "$BASE\temp.dta";

log using "$RD00\calories00_log", replace;
log on;

/* check to see which foods have kcal_per_gram information */
tab error_unit;
sum kcal_per_gram if error_unit==0;
sum kcal_per_gram if error_unit==1;
tab food if kcal_per_gram==. & error_unit==0;
tab food if kcal_per_gram==. & error_unit==1;
log off;

/**********************************************************************************/
/* Estimate average daily calorie availability by food using two methods          */
/* METHOD 1: using household unit data to convert directly into calories          */
/* METHOD 2: using household expenditure data and estimated prices                */
/**********************************************************************************/

gen yearly = frequency;
tab yearly, missing;
recode yearly  1=365  2=52  3=24  4=12  5=4  6=2  7=1;
assert yearly ~=.;
label var yearly "Annualization of frequency variable";

/* multiple grams available by percent edible */
replace grams = grams*per_edible;

/******************************************************************/
/* METHOD 1: calculate total daily calories available by food     	  	   */
/******************************************************************/

g calories = kcal_per_gram*grams if error_unit==0;
replace calories = 0 if food == 53;					/* assume purified water has zero calories */
replace calories = 0 if food == 57;					/* assume cigarettes have zero calories */
label variable calories "daily calories available (from unit data)";

/* daily calories available by food and household */
replace calories = (calories*yearly)/365 if error_unit==0;

/* create a label for records corresponding to foods with absolutely no calorie data */
bys food: egen calories_missing = sum((calories~=.));
replace calories_missing = -1 if calories_missing>0;
replace calories_missing = 1 if calories_missing==0;
replace calories_missing = 0 if calories_missing==-1;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_calories00", replace;

sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

/**********************************************************************************/
/* collapse data down to the household-by-food item level				    */
/**********************************************************************************/

collapse (sum) calories cons_yr error_unit calories_missing, by(i01 i02 comcens hogarid food);	

/* NOTE: (sum) collapse has the peculiar feature that if all units of missing in the 		 */
/*       group being summed over it will return a zero value and not a missing value 		 */
/*	   the calories_missing variable keeps track of such cases so that they can be 		 */
/*       properly set back equal to zero.										 */

/**********************************************************************************/
/* calculate unit level prices using median comarca calories per gram among 	    */
/* records with valid units, or muncipio or survey means (if needed)		    */
/**********************************************************************************/

g daily_exp = cons_yr/365;	
g t = calories/daily_exp if error_unit==0 & calories_missing==0;

bys comcens food: egen u1 = median(t);			/* median food-specific price in a comarca */
bys comcens food: egen c1 = count(t);			/* number of unit value observations in comarca */
bys i02 food: egen u2 = median(t);				/* median food-specific price in a municipio */	
bys i02 food: egen c2 = count(t);				/* number of unit value observations in municipio */
bys food: egen u3 = median(t);				/* median food-specific price in the survey */	
bys food: egen c3 = count(t);					/* number of unit value observations in survey */

g calorie_per_cordoba = u1 if c1>=10;
g str50 price_source = "comarca median" if c1>=10;
replace calorie_per_cordoba = u2 if calorie_per_cordoba ==. & c2>=10;
replace price_source = "municipio median" if c1<10 & c2>=10;
replace calorie_per_cordoba = u3 if calorie_per_cordoba ==.;
replace price_source = "survey median" if c1<10 & c2<10;
drop t u1 u2 u3 c1 c2 c3;
label variable calorie_per_cordoba "Median calories per cordoba within comarca-food cells";

log on;
/* distribution of price data by source */
tab price_source;
log off;

collapse (mean) calorie_per_cordoba, by(comcens food);
sort comcens food;
save "$RD00\g_calorie_prices00", replace; 

/* NOTE: since calorie data is only available (prior to imputation) for observations with valid units */
/*       this method should give sensible local price estimates (in principle)      	            */

/**********************************************************************************************/
/* impute calories for observations with invalid units using the calorie price data	          */
/**********************************************************************************************/

use "$RD00\g_calories00", clear;
sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

sort comcens food;
merge comcens food using "$RD00\g_calorie_prices00", uniqusing;
assert _merge==3;
drop _merge;

log on;
/* list foods without/with price data */
tab food if calorie_per_cordoba == .;
tab food if calorie_per_cordoba ~= .;
log off;

sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

g str80 source_of_calorie_measure = "valid unit data" if error_unit==0 & calories~=.;
replace calories = calorie_per_cordoba*value*(yearly/365) if error_unit==1 & calories==.;
replace source_of_calorie_measure = "imputed using value data" if error_unit==1 & calories~=.;

log on;
/* summarize results of calorie imputation procedure */
tab source_of_calorie_measure, missing;
tab food if calorie_per_cordoba==.;
log off;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_calories00", replace;

/*****************************************************************************************************************/
/* collapse data down to househould-by-food level and check for outliers in calorie estimates based on unit data */
/*****************************************************************************************************************/

collapse (sum) calories calories_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace calories = . if calories_missing>0;

sort hogarid; 
merge hogarid using "$RD00\g_demo00", nokeep;
assert _m == 3;
drop _m; 

/* calories per adult equivalent male by food */
g calories_ame=calories/adult_male_equivalent if calories_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(calories_ame), p(25);
bys food: egen c_75 = pctile(calories_ame), p(75);
bys food: egen c_iqr = iqr(calories_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g calories_outlier=0;
replace calories_outlier = 1 if calories_ame<lower_bd | calories_ame>upper_bd;
replace calories_outlier = . if calories_missing>0;
label variable calories_outlier "Apparent outlier in units-based calorie estimate";
drop calories_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab calories_outlier error_unit, missing;
log off;

/* for outlier households replace calories available with comarca, municipio or survey median */
/* food-specific calories available adjusted for household composition */

g calories_ame = calories/adult_male_equivalent if calories_outlier==0;
bys comcens food: egen u1 = median(calories_ame);
bys comcens food: egen c1 = count(calories_ame);
bys i02 food: egen u2 = median(calories_ame);
bys i02 food: egen c2 = count(calories_ame);
bys food: egen u3 = median(calories_ame);
bys food: egen c3 = count(calories_ame);

g calories_b	   = calories if calories_outlier==0;
g str80 source_of_calorie_measure_b = "valid unit/value data" if calories_outlier==0;
replace calories_b = u1*adult_male_equivalent if calories_outlier==1 & c1>=10;
replace source_of_calorie_measure_b = "adult male equivalent comarca median" if calories_outlier==0 & c1>=10;
replace calories_b = u2*adult_male_equivalent if calories_outlier==1 & calories_b==. & c1<10 & c2>=10;
replace source_of_calorie_measure_b = "adult male equivalent municipio median" if calories_outlier==1 & calories_b==. & c1<10 & c2>=10;
replace calories_b = u3*adult_male_equivalent if calories_outlier==1 & calories_b==.;
replace source_of_calorie_measure_b = "adult male equivalent survey median" if calories_outlier==1 & calories_b==.;
label variable calories_b "Outlier corrected daily calories available (from unit data)";
label variable source_of_calorie_measure_b "Source of outlier corrected calorie measure";
drop u1 u2 u3 c1 c2 c3 calories_ame;

/*****************************************************************/
/* Take outlier-corrected food specific calorie totals and       */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid calories_outlier source_of_calorie_measure_b food constype calories_b;
sort hogarid food constype; 
save "$RD00\toutlier", replace;

use "$RD00\g_calories00", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD00\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD00\toutlier.dta";

/**********************************************************************************************/
/* METHOD 2: estimate calorie availability using value data	 			                */
/**********************************************************************************************/

g calories_c = calorie_per_cordoba*(cons_yr/365);

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_calories00", replace;

/*****************************************************************************************************************/
/* collapse data down to household-by-food level and check for outliers in calorie estimates based on unit data 		     */
/*****************************************************************************************************************/

collapse (sum) calories_c calories_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace calories_c = . if calories_missing>0;

sort hogarid; 
merge hogarid using "$RD00\g_demo00", nokeep;
assert _m == 3;
drop _m; 

/* calories per adult equivalent male by food */
g calories_ame=calories_c/adult_male_equivalent if calories_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(calories_ame), p(25);
bys food: egen c_75 = pctile(calories_ame), p(75);
bys food: egen c_iqr = iqr(calories_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g calories_outlier_c=0;
replace calories_outlier_c = 1 if calories_ame<lower_bd | calories_ame>upper_bd;
replace calories_outlier_c = . if calories_missing>0;
label variable calories_outlier_c "Apparent outlier in values and price-based calorie estimate";
drop calories_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab calories_outlier_c error_unit, missing;
log off;

/* for outlier households replace calories available with comarca, municipio or survey median */
/* food-specific calories available adjusted for household composition */

g calories_ame = calories_c/adult_male_equivalent if calories_outlier_c==0;
bys comcens food: egen u1 = median(calories_ame);
bys comcens food: egen c1 = count(calories_ame);
bys i02 food: egen u2 = median(calories_ame);
bys i02 food: egen c2 = count(calories_ame);
bys food: egen u3 = median(calories_ame);
bys food: egen c3 = count(calories_ame);

g calories_d	   = calories_c if calories_outlier_c==0;
g str80 source_of_calorie_measure_d = "valid price/value data" if calories_outlier_c==0;
replace calories_d = u1*adult_male_equivalent if calories_outlier_c==1 & c1>=10;
replace source_of_calorie_measure_d = "adult male equivalent comarca median" if calories_outlier_c==0 & c1>=10;
replace calories_d = u2*adult_male_equivalent if calories_outlier_c==1 & calories_d==. & c1<10 & c2>=10;
replace source_of_calorie_measure_d = "adult male equivalent municipio median" if calories_outlier_c==1 & calories_d==. & c1<10 & c2>=10;
replace calories_d = u3*adult_male_equivalent if calories_outlier_c==1 & calories_d==.;
replace source_of_calorie_measure_d = "adult male equivalent survey median" if calories_outlier_c==1 & calories_d==.;
label variable calories_d "Outlier corrected daily calories available (from price/value data)";
label variable source_of_calorie_measure_d "Source of outlier corrected calorie measure";
drop u1 u2 u3 c1 c2 c3 calories_ame;

/*****************************************************************/
/* Take outlier-corrected food specific calorie totals and       */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid calories_outlier_c source_of_calorie_measure_d food constype calories_d;
sort hogarid food constype; 
save "$RD00\toutlier", replace;

use "$RD00\g_calories00", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD00\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD00\toutlier.dta";

keep 		hogarid comcens i01 i02 comarca i04 food constype calories 
		calories_b calories_c calories_d 
		source_of_calorie_measure source_of_calorie_measure_b source_of_calorie_measure_d calories_outlier calories_outlier_c
		cons_yr cons_yr_b;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_calories00", replace;

/***********************************************************************************/
/* compute household calorie per capita availability less meals out	           */
/***********************************************************************************/

/*******************************************************************/
/* use second calorie estimate (unit data with outlier correction) */
/* use outlier corrected expenditure data as well                  */
/*******************************************************************/

keep hogarid food calories_b cons_yr_b;
rename calories_b calories;
rename cons_yr_b cons_yr;

/* categories each food */
g cereals = 1 if inlist(food,1,2,3,4,5,6,7,8,9,10);
replace cereals = 0 if cereals==.;
g cereal_calories = cereals*calories;
g cereal_exp = cereals*(cons_yr/365);

g roots = 1 if inlist(food,31,39,43);
replace roots = 0 if roots==.;
g root_calories = roots*calories;
g root_exp = roots*(cons_yr/365);

g pulses = 1 if inlist(food,41);
replace pulses = 0 if pulses==.;
g pulse_calories = pulses*calories;
g pulse_exp = pulses*(cons_yr/365);

g veggies = 1 if inlist(food,30,32,33,34,35,36,37,38,40,42);
replace veggies = 0 if veggies==.;
g veggie_calories = veggies*calories;
g veggie_exp = veggies*(cons_yr/365);

g fruits = 1 if inlist(food,27,28,29);
replace fruits = 0 if fruits==.;
g fruit_calories = fruits*calories;
g fruit_exp = fruits*(cons_yr/365);

g meats = 1 if inlist(food,12,13,14,15,16,17,18,19);
replace meats = 0 if meats==.;
g meat_calories = meats*calories;
g meat_exp = meats*(cons_yr/365);

g dairy = 1 if inlist(food,20,21,22,23,24);
replace dairy = 0 if dairy==.;
g dairy_calories = dairy*calories;
g dairy_exp = dairy*(cons_yr/365);

g oils = 1 if inlist(food,25,26);
replace oils = 0 if oils==.;
g oil_calories = oils*calories;
g oil_exp = oils*(cons_yr/365);

g beverages = 1 if inlist(food,11,52,53,54,55);
replace beverages = 0 if beverages==.;
g beverage_calories = beverages*calories;
g beverage_exp = beverages*(cons_yr/365);

g meals = 1 if inlist(food,58,99);
replace meals = 0 if meals==.;
g meal_calories = meals*calories;
g meal_exp = meals*(cons_yr/365);

g misc = 1 if inlist(food,44,45,46,47,47,48,49,50,51,56,57,59,60);
replace misc = 0 if misc==.;
g misc_calories = misc*calories;
g misc_exp = misc*(cons_yr/365);

g other = 1 if inlist(food,61,62);
replace other = 0 if other==.;
g other_calories = other*calories;
g other_exp = other*(cons_yr/365);

g food_exp = (cons_yr/365);

/* sum annual food expenditures over each category for each household */
collapse (sum)  calories 	cereal_calories root_calories pulse_calories veggie_calories fruit_calories 
					meat_calories dairy_calories oil_calories beverage_calories meal_calories misc_calories other_calories
					cereal_exp root_exp pulse_exp veggie_exp fruit_exp 
					food_exp meat_exp dairy_exp oil_exp beverage_exp meal_exp misc_exp other_exp, by(hogarid);

label var cereal_calories 		"Total daily cereal calories available";
label var root_calories 		"Total daily root calories available";
label var pulse_calories 		"Total daily pulse calories available";
label var veggie_calories 		"Total daily veggie calories available";
label var fruit_calories 		"Total daily fruit calories available";
label var meat_calories 		"Total daily meat calories available";
label var dairy_calories 		"Total daily dairy calories available";
label var oil_calories 			"Total daily oil calories available";
label var beverage_calories 		"Total daily beverage calories available";
label var meal_calories 		"Total daily meals calories available";
label var misc_calories 		"Total daily misc calories available";
label var other_calories 		"Total daily other calories available";

sort hogarid;

/****************************************************************/
/* impute calories from prepared food, other food and meals out */
/****************************************************************/

g calories_per_cordoba = calories/(food_exp-meal_exp-other_exp);							/* average number of calories per C$ */
replace meal_calories=(2/3)*calories_per_cordoba*meal_exp if calories_per_cordoba>0;			/* imputed meal food calories (150% mark-up)*/ 
replace other_calories=calories_per_cordoba*other_exp if calories_per_cordoba>0;				/* imputed other food calories (no mark-up)*/ 

/* save household-level food consumption totals */
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

/************************************************************************************/
/* some households' food consumption may be entirely in the other & meals categories*/
/* in which case a calories_per_cordoba measure does not exist for them             */
/* use comarca medians to impute calorie availability for them.                     */
/************************************************************************************/

bys comcens: egen t = median(calories_per_cordoba);
replace meal_calories=(2/3)*t*meal_exp if calories_per_cordoba==0 | calories_per_cordoba==.;	/* imputed meal food calories (150% mark-up)*/ 
replace other_calories=t*other_exp if calories_per_cordoba==0 | calories_per_cordoba==.;		/* imputed other food calories (no mark-up)*/ 

replace calories = calories+meal_calories+other_calories;
drop t calories_per_cordoba;

/*****************************************************************/
/* calculate cordobas per calorie for each household             */
/*****************************************************************/

g calorie_price = 1000*food_exp/calories;
label variable calorie_price "C$ per 1000 calories";

g cereal_price =  1000*cereal_exp/cereal_calories;
label variable cereal_price "C$ per 1000 cereal calories";

g root_price =  1000*root_exp/root_calories;
label variable root_price "C$ per 1000 root calories";

g pulse_price =  1000*pulse_exp/pulse_calories;
label variable pulse_price "C$ per 1000 pulse calories";

g veggie_price =  1000*veggie_exp/veggie_calories;
label variable veggie_price "C$ per 1000 veggie calories";

g fruit_price =  1000*fruit_exp/fruit_calories;
label variable fruit_price "C$ per 1000 fruit calories";

g meat_price =  1000*meat_exp/meat_calories;
label variable meat_price "C$ per 1000 meat calories";

g dairy_price =  1000*dairy_exp/dairy_calories;
label variable dairy_price "C$ per 1000 dairy calories";

g oil_price =  1000*oil_exp/oil_calories;
label variable oil_price "C$ per 1000 oil calories";

g beverage_price =  1000*beverage_exp/beverage_calories;
label variable beverage_price "C$ 1000 per beverage calories";

g meal_price =  1000*meal_exp/meal_calories;
label variable meal_price "C$ per 1000 meal calories";

g misc_price =  1000*misc_exp/misc_calories;
label variable misc_price "C$ per 1000 misc calories";

g other_price =  1000*other_exp/other_calories;
label variable other_price "C$ per 1000 other calories";

g bmmo_calories = (beverage_calories + meal_calories + misc_calories + other_calories);
g bmmo_exp  = (beverage_exp + meal_exp + misc_exp + other_exp);
g bmmo_price =  1000*bmmo_exp/bmmo_calories;

label var bmmo_calories 	"Total daily beverage, meals, misc and other calories available";
label variable bmmo_price   "C$ per 1000 beverage, meals, misc and other calories";

keep hogarid 	calories 
			cereal_calories root_calories pulse_calories veggie_calories fruit_calories meat_calories dairy_calories beverage_calories oil_calories
			meal_calories misc_calories other_calories bmmo_calories
			food_exp
			cereal_exp root_exp pulse_exp veggie_exp fruit_exp meat_exp dairy_exp beverage_exp oil_exp
			meal_exp misc_exp other_exp bmmo_exp
			calorie_price
			cereal_price root_price pulse_price veggie_price fruit_price meat_price dairy_price beverage_price oil_price
			meal_price misc_price other_price bmmo_price;

sort hogarid;
merge hogarid using "$RD00\g_total00";
assert _merge==3;
drop _m;

g calories_ame = calories/adult_male_equivalent;
centile calories_ame, centile(0,10,25,50,75,90,100);
sort hogarid;

/* check for outliers in calorie aggregate */
egen c_25 = pctile(calories_ame), p(25);
egen c_75 = pctile(calories_ame), p(75);
egen c_iqr = iqr(calories_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;
g calories_outlier=0;
replace calories_outlier = 1 if calories_ame<lower_bd | calories_ame>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* apparent outliers in the aggregate calorie availability measure */
tab calories_outlier;
centile calories_ame if calories_outlier==0, centile(0,10,25,50,75,90,100);
log off;

/* calculate per capita calorie availability by expenditure quartile */
g total_exp_ame = total_exp/adult_male_equivalent;
egen c_25 = pctile(total_exp_ame), p(25);
egen c_50 = pctile(total_exp_ame), p(50);
egen c_75 = pctile(total_exp_ame), p(75);

sum calories_ame calorie_price if total_exp_ame<=c_25;
sum calories_ame calorie_price if total_exp_ame>c_25 & total_exp_ame<=c_50;
sum calories_ame calorie_price if total_exp_ame>c_50 & total_exp_ame<=c_75;
sum calories_ame calorie_price if total_exp_ame>c_75;

log close;

g year = 2000;
sort hogarid;
save "$RD00\g_cal_exp_clean00", replace;
