/******************************************************************************************************/
/* CALCULATE ZINC AVAILABILITY BY HOUSEHOLD IN 2000								      */
/******************************************************************************************************/

/* use a semicolon as the command delimiter */
#delimit ;

clear;

set matsize 800;
set memory 100m;

/******************************************************************************************************/
/* Input zinc mg per 100g as conversion factors for foods derived from "Tabla de 		  	      */
/* Composición de Alimentos de Centroamerica" accessed online at                                     */
/* (http://www.tabladealimentos.net/tca/TablaAlimentos/inicio.html). The data appendix provides full  */
/* details and sources.                                                                               */
/******************************************************************************************************/

use "$RD00\g_grams00", clear;
save "$RD00\g_zinc00", replace;

use "$BASE\calorie_content", clear;
g food = real(code_rps);
collapse zinc_mg per_edible, by(food);
sort food;
save "$BASE\temp", replace;

use "$RD00\g_zinc00", clear;
sort food;
merge food using "$BASE\temp";
tab _m;
bys _m: tab food, missing;
drop if _m==2;
drop _m;
erase "$BASE\temp.dta";

log using "$RD00\zinc00_log", replace;
log on;

/* check to see which foods have zinc information */
tab error_unit;
sum zinc_mg if error_unit==0;
sum zinc_mg if error_unit==1;
tab food if zinc_mg==. & error_unit==0;
tab food if zinc_mg==. & error_unit==1;
log off;

/**********************************************************************************/
/* Estimate average daily zinc availability by food using two methods  	          */
/* METHOD 1: using household unit data to convert directly into zinc              */
/* METHOD 2: using household expenditure data and estimated prices                */
/**********************************************************************************/

gen yearly = frequency;
tab yearly, missing;
recode yearly  1=365  2=52  3=24  4=12  5=4  6=2  7=1;
assert yearly ~=.;
label var yearly "Annualization of frequency variable";

/* multiple grams available by percent edible */
replace grams = grams*per_edible;

/******************************************************************/
/* METHOD 1: calculate total daily zinc available by food     	*/
/******************************************************************/

g zinc = zinc_mg*grams if error_unit==0;
replace zinc = 0 if food == 53;					/* assume purified water has zero zinc */
replace zinc = 0 if food == 57;					/* assume cigarettes have zero zinc */
label variable zinc "daily zinc (mg) available (from unit data)";

/* daily zinc available by food and household */
replace zinc = (zinc*yearly)/365 if error_unit==0;

/* create a label for records corresponding to foods with absolutely no zinc data */
bys food: egen zinc_missing = sum((zinc~=.));
replace zinc_missing = -1 if zinc_missing>0;
replace zinc_missing = 1 if zinc_missing==0;
replace zinc_missing = 0 if zinc_missing==-1;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_zinc00", replace;

sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

/**********************************************************************************/
/* collapse data down to the household-by-food item level		  	          */
/**********************************************************************************/

collapse (sum) zinc cons_yr error_unit zinc_missing, by(i01 i02 comcens hogarid food);	

/* NOTE: (sum) collapse has the peculiar feature that if all units of missing in the 		 */
/*       group being summed over it will return a zero value and not a missing value 		 */
/*	  the zinc_missing variable keeps track of such cases so that they can be 		 	 */
/*       properly set back equal to zero.										 */

/**********************************************************************************/
/* calculate unit level prices using median comarca zinc per gram among 	    */
/* records with valid units, or muncipio or survey means (if needed)		    */
/**********************************************************************************/

g daily_exp = cons_yr/365;	
g t = zinc/daily_exp if error_unit==0 & zinc_missing==0;

bys comcens food: egen u1 = median(t);		/* median food-specific price in a comarca */
bys comcens food: egen c1 = count(t);		/* number of unit value observations in comarca */
bys i02 food: egen u2 = median(t);			/* median food-specific price in a municipio */	
bys i02 food: egen c2 = count(t);			/* number of unit value observations in municipio */
bys food: egen u3 = median(t);			/* median food-specific price in the survey */	
bys food: egen c3 = count(t);				/* number of unit value observations in survey */

g zinc_per_cordoba = u1 if c1>=10;
g str50 price_source = "comarca median" if c1>=10;
replace zinc_per_cordoba = u2 if zinc_per_cordoba ==. & c2>=10;
replace price_source = "municipio median" if c1<10 & c2>=10;
replace zinc_per_cordoba = u3 if zinc_per_cordoba ==.;
replace price_source = "survey median" if c1<10 & c2<10;
drop t u1 u2 u3 c1 c2 c3;
label variable zinc_per_cordoba "Median zinc (g) per cordoba within comarca-food cells";

log on;
/* distribution of price data by source */
tab price_source;
log off;

collapse (mean) zinc_per_cordoba, by(comcens food);
sort comcens food;
save "$RD00\g_zinc_prices00", replace; 

/* NOTE: since zinc data is only available (prior to imputation) for observations with valid units 	 */
/*       this method should give sensible local price estimates (in principle)      			 */

/**********************************************************************************************/
/* impute zinc for observations with invalid units using the zinc price data	          	    */
/**********************************************************************************************/

use "$RD00\g_zinc00", clear;
sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

sort comcens food;
merge comcens food using "$RD00\g_zinc_prices00", uniqusing;
assert _merge==3;
drop _merge;

log on;
/* list foods without/with price data */
tab food if zinc_per_cordoba == .;
tab food if zinc_per_cordoba ~= .;
log off;

sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

g str80 source_of_zinc_measure = "valid unit data" if error_unit==0 & zinc~=.;
replace zinc = zinc_per_cordoba*value*(yearly/365) if error_unit==1 & zinc==.;
replace source_of_zinc_measure = "imputed using value data" if error_unit==1 & zinc~=.;

log on;
/* summarize results of zinc imputation procedure */
tab source_of_zinc_measure, missing;
tab food if zinc_per_cordoba==.;
log off;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_zinc00", replace;

/*****************************************************************************************************************/
/* collapse data down to househould-by-food level and check for outliers in zinc estimates based on unit data    */
/*****************************************************************************************************************/

collapse (sum) zinc zinc_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace zinc = . if zinc_missing>0;

sort hogarid; 
merge hogarid using "$RD00\g_demo00", nokeep;
assert _m == 3;
drop _m; 

/* zinc per adult equivalent male by food */
g zinc_ame=zinc/adult_male_equivalent if zinc_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(zinc_ame), p(25);
bys food: egen c_75 = pctile(zinc_ame), p(75);
bys food: egen c_iqr = iqr(zinc_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g zinc_outlier=0;
replace zinc_outlier = 1 if zinc_ame<lower_bd | zinc_ame>upper_bd;
replace zinc_outlier = . if zinc_missing>0;
label variable zinc_outlier "Apparent outlier in units-based zinc estimate";
drop zinc_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab zinc_outlier error_unit, missing;
log off;

/* for outlier households replace zinc available with comarca, municipio or survey median */
/* food-specific zinc available adjusted for household composition */

g zinc_ame = zinc/adult_male_equivalent if zinc_outlier==0;
bys comcens food: egen u1 = median(zinc_ame);
bys comcens food: egen c1 = count(zinc_ame);
bys i02 food: egen u2 = median(zinc_ame);
bys i02 food: egen c2 = count(zinc_ame);
bys food: egen u3 = median(zinc_ame);
bys food: egen c3 = count(zinc_ame);

g zinc_b	   = zinc if zinc_outlier==0;
g str80 source_of_zinc_measure_b = "valid unit/value data" if zinc_outlier==0;
replace zinc_b = u1*adult_male_equivalent if zinc_outlier==1 & c1>=10;
replace source_of_zinc_measure_b = "adult male equivalent comarca median" if zinc_outlier==0 & c1>=10;
replace zinc_b = u2*adult_male_equivalent if zinc_outlier==1 & zinc_b==. & c1<10 & c2>=10;
replace source_of_zinc_measure_b = "adult male equivalent municipio median" if zinc_outlier==1 & zinc_b==. & c1<10 & c2>=10;
replace zinc_b = u3*adult_male_equivalent if zinc_outlier==1 & zinc_b==.;
replace source_of_zinc_measure_b = "adult male equivalent survey median" if zinc_outlier==1 & zinc_b==.;
label variable zinc_b "Outlier corrected daily zinc available (from unit data)";
label variable source_of_zinc_measure_b "Source of outlier corrected zinc measure";
drop u1 u2 u3 c1 c2 c3 zinc_ame;

/*****************************************************************/
/* Take outlier-corrected food specific zinc totals and    	     */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid zinc_outlier source_of_zinc_measure_b food constype zinc_b;
sort hogarid food constype; 
save "$RD00\toutlier", replace;

use "$RD00\g_zinc00", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD00\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD00\toutlier.dta";

/**********************************************************************************************/
/* METHOD 2: estimate zinc availability using value data				                */
/**********************************************************************************************/

g zinc_c = zinc_per_cordoba*(cons_yr/365);

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_zinc00", replace;

/*****************************************************************************************************************/
/* collapse data down to household-by-food level and check for outliers in zinc estimates based on unit data     */
/*****************************************************************************************************************/

collapse (sum) zinc_c zinc_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace zinc_c = . if zinc_missing>0;

sort hogarid; 
merge hogarid using "$RD00\g_demo00", nokeep;
assert _m == 3;
drop _m; 

/* zinc per adult equivalent male by food */
g zinc_ame=zinc_c/adult_male_equivalent if zinc_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(zinc_ame), p(25);
bys food: egen c_75 = pctile(zinc_ame), p(75);
bys food: egen c_iqr = iqr(zinc_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g zinc_outlier_c=0;
replace zinc_outlier_c = 1 if zinc_ame<lower_bd | zinc_ame>upper_bd;
replace zinc_outlier_c = . if zinc_missing>0;
label variable zinc_outlier_c "Apparent outlier in values and price-based zinc estimate";
drop zinc_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab zinc_outlier_c error_unit, missing;
log off;

/* for outlier households replace zinc available with comarca, municipio or survey median */
/* food-specific zinc available adjusted for household composition */

g zinc_ame = zinc_c/adult_male_equivalent if zinc_outlier_c==0;
bys comcens food: egen u1 = median(zinc_ame);
bys comcens food: egen c1 = count(zinc_ame);
bys i02 food: egen u2 = median(zinc_ame);
bys i02 food: egen c2 = count(zinc_ame);
bys food: egen u3 = median(zinc_ame);
bys food: egen c3 = count(zinc_ame);

g zinc_d	   = zinc_c if zinc_outlier_c==0;
g str80 source_of_zinc_measure_d = "valid price/value data" if zinc_outlier_c==0;
replace zinc_d = u1*adult_male_equivalent if zinc_outlier_c==1 & c1>=10;
replace source_of_zinc_measure_d = "adult male equivalent comarca median" if zinc_outlier_c==0 & c1>=10;
replace zinc_d = u2*adult_male_equivalent if zinc_outlier_c==1 & zinc_d==. & c1<10 & c2>=10;
replace source_of_zinc_measure_d = "adult male equivalent municipio median" if zinc_outlier_c==1 & zinc_d==. & c1<10 & c2>=10;
replace zinc_d = u3*adult_male_equivalent if zinc_outlier_c==1 & zinc_d==.;
replace source_of_zinc_measure_d = "adult male equivalent survey median" if zinc_outlier_c==1 & zinc_d==.;
label variable zinc_d "Outlier corrected daily zinc (g) available (from price/value data)";
label variable source_of_zinc_measure_d "Source of outlier corrected zinc measure";
drop u1 u2 u3 c1 c2 c3 zinc_ame;

/*****************************************************************/
/* Take outlier-corrected food specific zinc totals and  	     */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid zinc_outlier_c source_of_zinc_measure_d food constype zinc_d;
sort hogarid food constype; 
save "$RD00\toutlier", replace;

use "$RD00\g_zinc00", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD00\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD00\toutlier.dta";

keep 		hogarid comcens i01 i02 comarca i04 food constype zinc 
		zinc_b zinc_c zinc_d 
		source_of_zinc_measure source_of_zinc_measure_b source_of_zinc_measure_d zinc_outlier zinc_outlier_c
		cons_yr cons_yr_b;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_zinc00", replace;

/***********************************************************************************/
/* compute household zinc per capita availability less meals out			     */
/***********************************************************************************/

/*******************************************************************/
/* use second zinc estimate (unit data with outlier correction) 	 */
/* use outlier corrected expenditure data as well                  */
/*******************************************************************/

keep hogarid food zinc_b cons_yr_b;
rename zinc_b zinc;
rename cons_yr_b cons_yr;

g meals = 1 if inlist(food,58,99);
replace meals = 0 if meals==.;
g meal_exp = meals*(cons_yr/365);

g other = 1 if inlist(food,61,62);
replace other = 0 if other==.;
g other_exp = other*(cons_yr/365);

g food_exp = (cons_yr/365);

/* sum annual food expenditures over each category for each household */
collapse (sum)  zinc meal_exp other_exp food_exp, by(hogarid);

sort hogarid;

/****************************************************************/
/* impute zinc from prepared food, other food and meals out     */
/****************************************************************/

g zinc_per_cordoba = zinc/(food_exp-meal_exp-other_exp);						/* average number of zinc g per C$ */
g meal_zinc=(2/3)*zinc_per_cordoba*meal_exp if zinc_per_cordoba>0;				/* imputed meal food zinc (150% mark-up)*/ 
g other_zinc=zinc_per_cordoba*other_exp if zinc_per_cordoba>0;					/* imputed other food zinc (no mark-up)*/ 

/* save household-level zinc consumption totals */
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

/************************************************************************************/
/* some households' food consumption may be entirely in the other & meals categories*/
/* in which case a zinc_per_cordoba measure does not exist for them             	*/
/* use comarca medians to impute zinc availability for them.                     	*/
/************************************************************************************/

bys comcens: egen t = median(zinc_per_cordoba);
replace meal_zinc=(2/3)*t*meal_exp if zinc_per_cordoba==0 | zinc_per_cordoba==.;			/* imputed meal food zinc (150% mark-up)*/ 
replace other_zinc=t*other_exp if zinc_per_cordoba==0 | zinc_per_cordoba==.;				/* imputed other food zinc (no mark-up)*/ 

replace zinc = zinc+meal_zinc+other_zinc;
drop t zinc_per_cordoba;

/*****************************************************************/
/* calculate cordobas per zinc for each household                */
/*****************************************************************/

g zinc_price = 100*food_exp/zinc;
label variable zinc_price "C$ per 100mg of zinc";


keep hogarid zinc zinc_price;
			
sort hogarid;
merge hogarid using "$RD00\g_total00";
assert _merge==3;
drop _m;

g zinc_ame = zinc/adult_male_equivalent;
centile zinc_ame, centile(0,10,25,50,75,90,100);
sort hogarid;

/* check for outliers in zinc aggregate */
egen c_25 = pctile(zinc_ame), p(25);
egen c_75 = pctile(zinc_ame), p(75);
egen c_iqr = iqr(zinc_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;
g zinc_outlier=0;
replace zinc_outlier = 1 if zinc_ame<lower_bd | zinc_ame>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* apparent outliers in the aggregate zinc availability measure */
tab zinc_outlier;
centile zinc_ame if zinc_outlier==0, centile(0,10,25,50,75,90,100);
log off;

/* calculate per capita zinc availability by expenditure quartile */
g total_exp_ame = total_exp/adult_male_equivalent;
egen c_25 = pctile(total_exp_ame), p(25);
egen c_50 = pctile(total_exp_ame), p(50);
egen c_75 = pctile(total_exp_ame), p(75);

sum zinc_ame zinc_price if total_exp_ame<=c_25;
sum zinc_ame zinc_price if total_exp_ame>c_25 & total_exp_ame<=c_50;
sum zinc_ame zinc_price if total_exp_ame>c_50 & total_exp_ame<=c_75;
sum zinc_ame zinc_price if total_exp_ame>c_75;

log close;

g year = 2000;
sort hogarid;
save "$RD00\g_znc_exp_clean00", replace;
