/***********************************************************************************************/
/* Bryan S. Graham, UC - Berkeley (bgraham@econ.berkeley.edu)					     		   */
/* Jan 2008														     						   */
/***********************************************************************************************/	

/***********************************************************************************************/
/* This file is a adaptation of the "consumo02.do" file provided with the public release of the*/
/* IFPRI-RPS evaluation dataset. It constructs household-level consumption aggregates for 2001 */
/* using the IFPRI-RPS evaluation dataset. The construction of the aggregate closely follows   */
/* the guidelines advocated by Deaton and Zaidi (1992).   						     		   */
/***********************************************************************************************/                                                              

/* use a semicolon as the command delimiter */
#delimit ;

clear;

set matsize 800;
set memory 100m;

/***********************************************************************************************/
/* DATA FILES USED TO CONSTRUCT AGGREGATES								     				   */	
/* 															     							   */ 
/*     gastos_a01.dta												     					   */
/*     gastos_b101.dta												     					   */
/*     gastos_b201.dta												     				       */		
/*     gastos_b301.dta												     					   */
/*     gastos_b401.dta												     					   */	
/*     viviendalb01.dta												     					   */	
/*     poblacionlb01.dta											     					   */
/*     equipos01.dta 												     					   */
/***********************************************************************************************/	

/***********************************************************************************************/
/* DATA FILES CREATED BY THE PROGRAM									     				   */
/*   															     						   */	
/*     g_food01   	   (Annual food consumption including components)			     		   */
/*     g_nonfood01 	   (Annual non-food consumption including components)			     	   */		
/*     g_total01       	   (Consumption totals)								     			   */
/***********************************************************************************************/

/* DEFINE NOMINAL EXCHANGE RATE AS OF OCTOBER 2001 AS A GLOBAL VARIABLE */
global EXRATE "13.65";

/* generate file with basic household identification information */
use hogarid i01 i02 comarca i04 using "$RD01\viviendalb01"; 
sort hogarid; 
save "$RD01\tmpdigit" , replace;

/***********************************************************************************************/	
/* COMPUTATION OF FOOD CONSUMPTION AGGREGATE (SECTION 8.A and portion of SECTION 8.B)	     */
/***********************************************************************************************/	

/***********************************************************************************************/	
/* EXPENDITURES ON FOOD, DRINK & TOBACCO IN THE LAST 15 DAYS, SECTION 8.A			     */
/***********************************************************************************************/	

use "$RD01\gastos_a01",clear;								   /* load raw data */
keep if  s8ap03==1;									   /* keep records with non-zero purchases */

/* keep household id, food type, freq of purchase, units and total amount paid */
keep  hogarid s8ap00 s8ap04 s8ap05a s8ap05b s8ap06;
rename s8ap00  food;
rename s8ap04  frequency;
rename s8ap05a quantity;
rename s8ap05b unit;
rename s8ap06  value;

gen byte constype=6;						  			   /* record corresponds to purchased food      */
save "$RD01\tempfood", replace;					  		   /* save data */

/***********************************************************************************************/	
/* CONSUMPTION OF FOOD, DRINK & TOBACCO IN THE LAST 15 DAYS FROM NONMARKET SOURCE, SECTION 8.A */
/***********************************************************************************************/	

use "$RD01\gastos_a01"; 												  /* load raw data */
keep if  s8ap07~=6;									 /* keep records with non-zero acquisition */

/* keep household id, food type, freq of acquisition, units and total value recieved */
keep hogarid s8ap00 s8ap08 s8ap09a s8ap09b s8ap10 s8ap07;
order hogarid s8ap00 s8ap08 s8ap09a s8ap09b s8ap10 s8ap07;
rename s8ap00  food;
rename s8ap08  frequency;
rename s8ap09a quantity;
rename s8ap09b unit;
rename s8ap10  value;
rename s8ap07  constype;

append using "$RD01\tempfood";					     /* append records to purchased food data */

/* find households with no food expenditures recorded in SECTION 8.A */
gen boughtfood=1 if constype==6;
egen no_bought_food=mean(boughtfood), by (hogarid);
replace no_bought_food = 0 if no_bought_food==1;
replace no_bought_food = 1 if no_bought_food==.;
label var no_bought_food "Household has no recorded food expenditures in SECTION 8.A";
drop boughtfood;

sort hogarid; 
merge hogarid using "$RD01\tmpdigit", nokeep;
assert _m == 3;
drop _m; 

/* find households with no own production of food recorded in SECTION 8.A */
gen producefood=1 if constype==1;
egen no_produced_food=mean(producefood), by (hogarid);
replace no_produced_food = 0 if no_produced_food==1;
replace no_produced_food = 1 if no_produced_food==.;
label var no_produced_food "Household has no record of own production in SECTION 8.A";
drop producefood;

sort hogarid; 
by hogarid : gen first = _n;

/* list those households with no food expenditure, but some own production */
li i01 i02 comarca i04 hogarid if (no_bought_food == 1 & no_produced_food ==0 & first == 1) , noobs nolab;

/* list those households with no data on food expenditure & production */
li i01 i02 comarca i04 hogarid if (no_bought_food == 1 & no_produced_food == 1 & first == 1) , noobs nolab;

tab no_bought_food no_produced_food; 

save "$RD01\tempfood", replace;

/***********************************************************************************************/	
/* CONSUMPTION OF FOOD, DRINK & TOBACCO OUTSIDE OF THE HOME IN THE LAST WEEK, SECTION 8.B.1    */
/***********************************************************************************************/	

use hogarid s8b1p00 s8b1p03 s8b1p04 using "$RD01\gastos_b101";
keep if s8b1p00==5 & s8b1p03==1;   /* keep records of households with outside food consumption */ 

g food = 99;
g frequency  = 2;
rename s8b1p04 value;
g constype = 99;
drop s8b1p00 s8b1p03; 

/* append outside food records to other food records */
save "$RD01\outsidefood", replace; 
use "$RD01\tempfood";
append using "$RD01\outsidefood";

/* relabel value definitions for constype (s8ap07) variable */
label define s8ap07 	1 "Own production (8.A)" 
				2 "Part of pay (8.A)" 
				3 "Own store/business (8.A)" 
				4 "Gift (8.A)" 
				5 "Other (8.A)" 
				6 "Purchased for in home (8.A)"
				99 "Purchased for outside home (8.B.1)", modify;

drop no_bought_food no_produced_food;
sort hogarid;
format food %10.0f;
save "$RD01\tempfood", replace; 
erase "$RD01\outsidefood.dta";

/***********************************************************************************************/	
/* ESTIMATE ANNUAL FOOD CONSUMPTION FOR EACH HOUSEHOLD                                         */
/***********************************************************************************************/	

gen yearly = frequency;
tab yearly, missing;
recode yearly  1=365  2=52  3=24  4=12  5=4  6=2  7=1;
assert yearly ~=.;
label var yearly "Annualization of frequency variable";

/* compute annual food consumption in Cordobas and US$ using EXRATE */
g cons_yr=yearly*value;
label var cons_yr "Annual food consumption by food type/source in C$ (Section 8.A)";
g cons_yr_US = cons_yr/$EXRATE;
label var cons_yr_US "Annual food consumption by food type/source in US$ $EXRATE (Section 8.A)";

keep hogarid food quantity frequency unit value constype cons_yr cons_yr_US ;
compress; 

/**********************************************************************************/
/* Collapse data down to food-type level, check for outliers and make corrections */
/**********************************************************************************/

/* save data in current configuration */ 
save "$RD01\g_food01", replace;

sort hogarid food;
collapse (sum) cons_yr (max) constype, by(hogarid food);

sort hogarid; 
merge hogarid using "$RD01\g_demo01", nokeep;
assert _m == 3;
drop _m; 

g cons_yr_ame = cons_yr/adult_male_equivalent;			/* yearly consumption per adult male equivalent */

/* check for demographic-corrected consumption values 3 times the IQR above the 75th quantile and below the 25th quantile */
bys food: egen c_25 = pctile(cons_yr_ame), p(25);		/* 25th quantile of consumption by food */
bys food: egen c_75 = pctile(cons_yr_ame), p(75);		/* 75th quantile of consumption by food */
bys food: egen c_iqr = iqr(cons_yr_ame);				/* inter-quartile range */
g lower_bd = c_25-3*c_iqr;
g upper_bd = c_75+3*c_iqr;
g food_exp_outlier=0;
replace food_exp_outlier = 1 if cons_yr_ame<lower_bd | cons_yr_ame>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;
tab food_exp_outlier;

/* generate alternative measure of annual consumption where outliers have been replaced with census comarca medians (if available) */
/* otherwise using municipio or survey medians																		    	*/	
replace cons_yr_ame=. if food_exp_outlier==1;	
bys comcens food: egen u1 = median(cons_yr_ame);
bys comcens food: egen c1 = count(cons_yr_ame);
bys i01 i02 food: egen u2 = median(cons_yr_ame);
bys i01 i02 food: egen c2 = count(cons_yr_ame);
bys food: egen u3 = median(cons_yr_ame);
bys food: egen c3 = count(cons_yr_ame);
g	cons_yr_b = cons_yr if food_exp_outlier==0;	
replace cons_yr_b = u1*adult_male_equivalent if food_exp_outlier==1 & c1>=10;
replace cons_yr_b = u2*adult_male_equivalent if food_exp_outlier==1 & cons_yr_b==. & c2>=10;
replace cons_yr_b = u3*adult_male_equivalent if food_exp_outlier==1 & cons_yr_b==.;
drop u1 u2 u3 c1 c2 c3 cons_yr_ame;
g cons_yr_US_b = cons_yr_b/$EXRATE;

label var cons_yr_b "Annual food consumption by food type in C$ with outlier corrections (Section 8.A)";
label var cons_yr_US_b "Annual food consumption by food type in US$ $EXRATE with outlier corrections (Section 8.A)";

/*****************************************************************/
/* Take outlier-corrected food specific consumption totals and   */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid food_exp_outlier food constype cons_yr_b cons_yr_US_b;
sort hogarid food constype; 
save "$RD01\toutlier", replace;

use "$RD01\g_food01", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD01\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;

keep hogarid food quantity frequency unit value constype cons_yr cons_yr_US cons_yr_b cons_yr_US_b food_exp_outlier;

label data "LdB 2001 : Annual food consumption -- components";
sort hogarid food; 
save "$RD01\g_food01", replace;
erase "$RD01\toutlier.dta";

/*******************************************************************************/
/* Calculate aggregate food consumption and subtotals by food group     	 */
/*******************************************************************************/

g number_foods = 1;

/* categories each food */
g cereals = 1 if inlist(food,1,2,3,4,5,6,7,8,9,10);
replace cereals = 0 if cereals==.;
replace cereals = cereals*cons_yr;
g cereals_b = cereals*cons_yr_b;

g roots = 1 if inlist(food,31,39,43);
replace roots = 0 if roots==.;
replace roots = roots*cons_yr;
g roots_b = roots*cons_yr_b;

g pulses = 1 if inlist(food,41);
replace pulses = 0 if pulses==.;
replace pulses = pulses*cons_yr;
g pulses_b = pulses*cons_yr_b;

g veggies = 1 if inlist(food,30,32,33,34,35,36,37,38,40,42);
replace veggies = 0 if veggies==.;
replace veggies = veggies*cons_yr;
g veggies_b = veggies*cons_yr_b;

g fruits = 1 if inlist(food,27,28,29);
replace fruits = 0 if fruits==.;
replace fruits = fruits*cons_yr;
g fruits_b = fruits*cons_yr_b;

g meats = 1 if inlist(food,12,13,14,15,16,17,18,19);
replace meats = 0 if meats==.;
replace meats = meats*cons_yr;
g meats_b = meats*cons_yr_b;

g dairy = 1 if inlist(food,20,21,22,23,24);
replace dairy = 0 if dairy==.;
replace dairy = dairy*cons_yr;
g dairy_b = dairy*cons_yr_b;

g oils = 1 if inlist(food,25,26);
replace oils = 0 if oils==.;
replace oils = oils*cons_yr;
g oils_b = oils*cons_yr_b;

g beverages = 1 if inlist(food,11,52,53,54,55);
replace beverages = 0 if beverages==.;
replace beverages = beverages*cons_yr;
g beverages_b = beverages*cons_yr_b;

g meals = 1 if inlist(food,58,99);
replace meals = 0 if meals==.;
replace meals = meals*cons_yr;
g meals_b = meals*cons_yr_b;

g misc = 1 if inlist(food,44,45,46,47,47,48,49,50,51,56,57,59,60);
replace misc = 0 if misc==.;
replace misc = misc*cons_yr;
g misc_b = misc*cons_yr_b;

g other = 1 if inlist(food,61,62);
replace other = 0 if other==.;
replace other = other*cons_yr;
g other_b = other*cons_yr_b;


/* sum annual food expenditures over each category for each household */
collapse (sum)  food_exp_outlier number_foods total_food_cons_yr = cons_yr total_food_cons_yr_US = cons_yr_US 
		total_food_cons_yr_b = cons_yr_b total_food_cons_yr_US_b = cons_yr_US_b 
		cereals cereals_b roots roots_b pulses pulses_b veggies veggies_b fruits fruits_b meats meats_b dairy dairy_b oils oils_b 
		beverages beverages_b meals meals_b misc misc_b other other_b, by(hogarid);

label var food_exp_outlier "Number of food-level records with apparent outlier information";
label var number_foods "Total number of foods consumed by the household";

label var total_food_cons_yr "Total annual food consumption in C$";
label var total_food_cons_yr_US "Total annual food consumption in US$ $EXRATE";
label var total_food_cons_yr_b "Total annual food consumption in C$ with outlier corrections";
label var total_food_cons_yr_US_b "Total annual food consumption in US$ $EXRATE with outlier corrections";

label var cereals "Total annual cereal consumption in C$";
label var roots "Total annual root, tuber and plantain consumption in C$";
label var pulses "Total annual bean consumption in C$";
label var veggies "Total annual vegetable consumption in C$";
label var fruits "Total annual fruit consumption in C$";
label var meats "Total annual meat/fish consumption in C$";
label var dairy "Total annual dairy/egg consumption in C$";
label var oils "Total annual oil consumption in C$";
label var beverages "Total annual beverage consumption in C$";
label var meals "Total annual prepared food and meals out consumption in C$";
label var misc "Total annual sugars, sweets and condiments food consumption in C$";
label var other "Total annual other food consumption in C$";

label var cereals_b "Total annual cereal consumption in C$ w/ outlier corrections";
label var roots_b "Total annual root, tuber and plantain consumption in C$ w/ outlier corrections";
label var pulses_b "Total annual bean consumption in C$ w/ outlier corrections";
label var veggies_b "Total annual vegetable consumption in C$ w/ outlier corrections";
label var fruits_b "Total annual fruit consumption in C$ w/ outlier corrections";
label var meats_b "Total annual meat/fish consumption in C$ w/ outlier corrections";
label var dairy_b "Total annual dairy/egg consumption in C$ w/ outlier corrections";
label var oils_b "Total annual oil consumption in C$ w/ outlier corrections";
label var beverages_b "Total annual beverage consumption in C$ w/ outlier corrections";
label var meals_b "Total annual prepared food and meals out consumption in C$ w/ outlier corrections";
label var misc_b "Total annual sugars, sweets and condiments food consumption in C$ w/ outlier corrections";
label var other_b "Total annual other food consumption in C$ w/ outlier corrections";
sort hogarid;

/* save household-level food consumption totals */
merge hogarid using "$RD01\g_demo01";
assert _merge==3;
drop _m;

keep 	i01 i02 comarca i04 comcens hogarid food_exp_outlier number_foods
	total_food_cons_yr total_food_cons_yr_b total_food_cons_yr_US total_food_cons_yr_US_b 
	cereals roots pulses veggies fruits meats dairy oils beverages meals misc other
	cereals_b roots_b pulses_b veggies_b fruits_b meats_b dairy_b oils_b beverages_b meals_b misc_b other_b
	caloric_requirement member non_member adult_male_equivalent head_male spouse head_yrssch spouse_yrssch 
	age_0t1g age_0t1g_n age_0t1b age_0t1b_n age_1t5g age_1t5g_n age_1t5b age_1t5b_n age_5t9g age_5t9g_n age_5t9b age_5t9b_n 
	age_9t14g age_9t14g_n age_9t14b age_9t14b_n age_14t18g age_14t18g_n age_14t18b age_14t18b_n 
	age_18t30g age_18t30g_n age_18t30b age_18t30b_n age_30t60g age_30t60g_n age_30t60b age_30t60b_n age_60g age_60g_n age_60b age_60b_n;

sort hogarid;
save "$RD01\tfood", replace;

/* calculate annual food consumption per capita */
g food_pc = total_food_cons_yr/member;
label var food_pc "Annual food consumption per capita in C$";
g food_pc_US = total_food_cons_yr_US/member;
label var food_pc_US "Annual food consumption per capita in US$ $EXRATE";

g food_pc_b = total_food_cons_yr/member;
label var food_pc_b "Annual food consumption per capita in C$ with outlier corrections";
g food_pc_US_b = total_food_cons_yr_US/member;
label var food_pc_US_b "Annual food consumption per capita in US$ $EXRATE with outlier corrections";

sort hogarid;
save "$RD01\tfood", replace;
erase "$RD01\tempfood.dta";
use "$RD01\g_food01" , clear;  
drop cons_yr_US cons_yr_US_b;
save "$RD01\g_food01", replace;

/***********************************************************************************************/	
/* COMPUTATION OF NON-FOOD CONSUMPTION AGGREGATE (SECTION 8.B)			  		     		   */
/***********************************************************************************************/	

/***********************************************************************************************/	
/* This aggregate is based on information contained in four data files  			     	   */
/* (1) gastos_b1 (weekly), (2) gastos_b2 (monthly), gastos_b3 (six months), gastos_b4 (annual) */
/***********************************************************************************************/	

clear;

/* non-food weekly expenditures */
use "$RD01\gastos_b101", clear;
drop if s8b1p00==5; 				/* only keep non-food expenditures */
gen cons_yr = s8b1p04*52;			/* annualize expenditures */
rename s8b1p00 constype;
keep hogarid constype cons_yr;
save "$RD01\temp1", replace;

/* non-food monthly expenditures */
use "$RD01\gastos_b201", clear;
gen cons_yr = s8b2p02*12;			/* annualize expenditures */
rename s8b2p00 constype;
keep hogarid constype cons_yr;
replace constype = constype+5;		/* renumber constype to avoid overlap w/ weekly expenditures */
save "$RD01\temp2", replace;

/* non-food bi-annual expenditures */
use "$RD01\gastos_b301", clear;
gen cons_yr = s8b3p02*2;			/* annualize expenditures */
rename s8b3p00 constype;
keep hogarid constype cons_yr;
replace constype = constype+30;		/* renumber constype to avoid overlaps w/ weekly & monthly expenditures */
save "$RD01\temp3", replace;

/* non-food annual expenditures */

/***********************************************************************/
/* NOTE: Items 1-4, 8-9, 11 are consumer durables. Item 13 is taxes.   */
/* Item 15 is religious ceremonies/weddings. Item 18 is farm equipment */
/* Deaton & Zaidi (2002) recommend excluding these from the consumption*/ 
/* aggregate and we do so here.					           */
/***********************************************************************/

use "$RD01\gastos_b401", clear;
rename s8b4p02 cons_yr;
rename s8b4p00 constype;
keep hogarid constype cons_yr;
drop if inlist(constype,1,2,3,4,8,9,11,13,15,18);		    /* drop lumpy expenditures from non-food totals */ 
replace constype = constype+44;		  			    /* renumber constype to avoide overlap with weekly, monthly & bi-annual expenditures */

/* combine the four annualized non-food expenditure files */
append using "$RD01\temp1";
append using "$RD01\temp2";
append using "$RD01\temp3";
drop if cons_yr==.;

/* calculate non-food expenditures in US dollars at $EXRATE */
gen cons_yr_US=cons_yr/$EXRATE;
label var cons_yr  "Annual non-food expenditure by type in C$ (Section 8.B)";
label var cons_yr_US  "Annual non-food expenditure by type in US$ $EXRATE (Section 8.B)";
rename constype constype_nf;
label var constype_nf "Non-food consumption type";
label drop _all;

/*******************************************************************************/
/* Check for outliers and make corrections		   			       */
/*******************************************************************************/

sort hogarid; 
merge hogarid using "$RD01\g_demo01", nokeep;
assert _m == 3;
drop _m; 

g cons_yr_ame = cons_yr/adult_male_equivalent;			/* yearly consumption per adult male equivalent */

/* check for per capita consumption values 3 times the IQR above the 75th quantile and below the 25th quantile */
bys constype_nf: egen c_25 = pctile(cons_yr_ame), p(25);		/* 25th quantile of consumption by food */
bys constype_nf: egen c_75 = pctile(cons_yr_ame), p(75);		/* 75th quantile of consumption by food */
bys constype_nf: egen c_iqr = iqr(cons_yr_ame);				/* inter-quartile range */
g lower_bd = c_25-3*c_iqr;
g upper_bd = c_75+3*c_iqr;
g nffood_exp_outlier=0;
replace nffood_exp_outlier = 1 if cons_yr_ame<lower_bd | cons_yr_ame>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;
tab nffood_exp_outlier;

/* generate alternative measure of annual consumption where outliers have been replaced with census comarca medians */
replace cons_yr_ame=. if nffood_exp_outlier==1;
bys comcens constype_nf: egen u1 = median(cons_yr_ame);
bys comcens constype_nf: egen c1 = count(cons_yr_ame);
bys i01 i02 constype_nf: egen u2 = median(cons_yr_ame);
bys i01 i02 constype_nf: egen c2 = count(cons_yr_ame);
bys i01 constype_nf: egen u3 = median(cons_yr_ame);
bys i01 constype_nf: egen c3 = count(cons_yr_ame);

g	cons_yr_b = cons_yr if nffood_exp_outlier==0;	
replace cons_yr_b = u1*adult_male_equivalent if nffood_exp_outlier==1 & c1>=10;
replace cons_yr_b = u2*adult_male_equivalent if nffood_exp_outlier==1 & cons_yr_b==. & c2>=10;
replace cons_yr_b = u3*adult_male_equivalent if nffood_exp_outlier==1 & cons_yr_b==.;
drop u1 u2 u3 c1 c2 c3 cons_yr_ame;
g cons_yr_US_b = cons_yr_b/$EXRATE;

label var cons_yr_b  "Annual non-food expenditure by type in C$ with outlier corrections (Section 8.B)";
label var cons_yr_US_b  "Annual non-food expenditure by type in US$ $EXRATE with outlier corrections (Section 8.B)";

keep hogarid nffood_exp_outlier constype_nf cons_yr cons_yr_US cons_yr_b cons_yr_US_b;

label data "LdB 2001 : Annual non-food consumption - components";
compress; 
sort hogarid constype_nf;
save "$RD01\g_nonfood01", replace;

/******************************************************/
/* Check non-food consumption aggregates for outliers */
/******************************************************/

g number_nonfood = 1;

/* sum expenditures of all types for each household */
collapse (sum)  nffood_exp_outlier number_nonfood total_nonfood_cons_yr=cons_yr total_nonfood_cons_yr_US=cons_yr_US 
		total_nonfood_cons_yr_b=cons_yr_b total_nonfood_cons_yr_US_b=cons_yr_US_b, by(hogarid);

label var nffood_exp_outlier "Number of nonfood expenditures that are apparent outliers";
label var number_nonfood "Number of nonfood expenditures in the household";

label var total_nonfood_cons_yr  "Total annual non-food expenditures in C$ (Section 8.B)";
label var total_nonfood_cons_yr_US  "Total annual non-food expenditures in US$ $EXRATE (Section 8.B)";
label var total_nonfood_cons_yr_b  "Total annual non-food expenditures in C$ with outlier corrections (Section 8.B)";
label var total_nonfood_cons_yr_US_b  "Total annual non-food expenditures in US$ $EXRATE with outlier corrections (Section 8.B)";

sort hogarid; 
save "$RD01\tnofood", replace;

erase "$RD01\temp1.dta";
erase "$RD01\temp2.dta";
erase "$RD01\temp3.dta";

rename total_nonfood_cons_yr nonfood;
rename total_nonfood_cons_yr_US nonfood_US;
rename total_nonfood_cons_yr_b nonfood_b;
rename total_nonfood_cons_yr_US_b nonfood_US_b;

sort hogarid;
merge hogarid using "$RD01\tfood";		/* merge with food consumption aggregates */
tab _merge;
assert _merge~=1;
drop _merge;
recode nonfood      .=0;			/* set missing expenditures to zero */
recode nonfood_US   .=0; 
recode nonfood_b    .=0;				
recode nonfood_US_b .=0; 

rename total_food_cons_yr food;
rename total_food_cons_yr_US food_US;
rename total_food_cons_yr_b food_b;
rename total_food_cons_yr_US_b food_US_b;

/* check/identify apparently erroneous consumption aggregates */

egen avg=mean(nonfood_US);					/* average non-food expenditures */
gen error_nf=1 if nonfood_US==0;				/* households with zero non-food expenditures */
replace error_nf=2 if (avg/nonfood_US>10);		/* households with one-tenth of average */
replace error_nf=3 if (nonfood_US/avg>10);		/* households with ten times average */
replace error_nf=1 if nonfood_US==0;
label define error_nf 1 "Non-food=0" 2 "Non-Food is 10x<avg" 3 "Non-Food is 10x>avg";
label values error_nf error_nf;
label var error_nf "Possible errors in non-food expenditure data";

drop avg;
egen avg=mean(food_US);					/* average food expenditures */
gen error_f=1 if (avg/food_US>10);			/* households with one-tenth of average */
replace error_f=2 if (food_US/avg>10);		/* households with time times average */
label define error_f 1 "Food is 10x<avg" 2 "Food is 10x>avg";
label values error_f error_f;
label var error_f "Possible errors in food expenditure data";
drop avg;

gen total   = nonfood + food;				

order hogarid food nonfood food_US nonfood_US error_nf error_f;
gen error_t=1 if total==.;					
drop total food_US food_US_b food_pc_US nonfood_US nonfood_US_b;
label var error_t "Household reports zero expenditures in Sections 8.A & 8.B";

sort hogarid;
save "$RD01\g_total01", replace;

erase "$RD01\tfood.dta";
erase "$RD01\tnofood.dta";

/* save annualized components of non-food expenditure */
use "$RD01\g_nonfood01"; 			
format constype_nf %5.0f; 
format cons_yr %9.0g;
format cons_yr_b %9.0g;
drop cons_yr_US cons_yr_US_b; 
save "$RD01\g_nonfood01", replace; 

/***********************************************************************************************/	
/* COMPUTATION OF HOUSING CONSUMPTION AGGREGATE	(SECTION 3)						     		   */
/***********************************************************************************************/

use hogarid s3ap03a s3ap03b s3ap04 s3ap05 s3ap08a s3ap08b s3ap11a s3ap11b s3ap14a s3ap14b s3ap15a s3ap15b s3ap19 s3ap21 using "$RD01\viviendalb01", clear;

rename s3ap03a rent1;							/* Actual rent */
rename s3ap05  rent2;							/* Household's self-reported implicit rental value */	
rename s3ap08a water;							/* Last water bill */
rename s3ap11a garbage;							/* Last garbage bill */
rename s3ap14a electricity;						/* Last electricity bill */
rename s3ap15a lighting;						/* Last lighting fuel bill */
rename s3ap19  cooking;							/* Last cooking fuel bill */
rename s3ap21  telephone;						/* Last telephone bill */

/* calculate annual rent -- either actual or implicit */
recode s3ap03b 1=52 2=26 3=12 4=4 5=2 6=1;
replace rent1=rent1*s3ap03b;						/* Annual rent (actual) */					
replace rent2=rent2*12;							/* Annual rent (implicit) */	
recode rent1 .=0; 							/* recode missing rents to zero */
recode rent2 .=0;
gen rent = rent1 + rent2;						/* Annual rent is actual if available and implicit otherwise */ 

/* calculate annual water expenditure */
recode s3ap08b 1=52 2=26 3=12 4=4 5=2 6=1;
replace water = water*s3ap08b;

/* calculate annual garbage expenditure */
recode s3ap11b 1=52 2=26 3=12 4=4 5=2 6=1;
replace garbage=garbage*s3ap11b;

/* calculate annual electricity expenditure */
recode s3ap14b 1=52 2=26 3=12 4=4 5=2 6=1;
replace electricity = electricity*s3ap14b;

/* calculate annual lighting fuel expenditure */
recode s3ap15b 1=52 2=26 3=12 4=4 5=2 6=1;
replace lighting=lighting*s3ap15b;

/* calculate annual cooking fuel expenditure */
replace cooking=cooking*12;

/* calculate annual telephone expenditure */
replace telephone=telephone*12;

recode water     	.=0;
recode garbage   	.=0;
recode electricity	.=0;
recode lighting   	.=0;
recode cooking  	.=0;
recode telephone	.=0;

sort hogarid;
save "$RD01\g_other", replace;

use "$RD01\g_total01"; 
merge hogarid using "$RD01\g_other";
assert _m==3; 
drop _m s3ap14b s3ap03b s3ap04 rent1 rent2 s3ap08b s3ap11b s3ap15b;
label var rent "Actual or implicit annual rent";

sort hogarid;
save "$RD01\g_total01" , replace;

/***********************************************************************************************/	
/* COMPUTATION OF HEALTH CONSUMPTION AGGREGATE (SECTION 4)						     */
/***********************************************************************************************/

use hogarid s4p28b s4p29b s4p36 s4p39b s4p43b s4p44b s4p46b s4p47 using "$RD01\poblacionlb01";

gen health=s4p47*12;

/* some households reported only "one-time expenditures" -- do not multiple their totals by 12 */
/* NOTE: This adjustmented is as made by RPS-IFPRI evaluation team 	                       */

replace health=s4p47 if	hogarid=="00097801"   |hogarid=="00129201"   |hogarid=="00296401"   |hogarid=="00320601"   |hogarid=="00321001"   |
				hogarid=="00332201"   |hogarid=="00531901"   |hogarid=="00638901"   |hogarid=="00671601"   |hogarid=="00697701"   |
				hogarid=="00747001"   |hogarid=="00773701"   |hogarid=="00775201"   |hogarid=="00820101"   |hogarid=="00847401"   |
				hogarid=="00945801"   |hogarid=="00946701";   

recode health .=0;

/* total annual health spending for each household */
collapse (sum) health s4p28b s4p29b s4p36 s4p39b s4p43b s4p44b s4p46b, by (hogarid);
keep health hogarid; 
sort hogarid;
save "$RD01\g_health", replace;

use "$RD01\g_total01";
merge hogarid using "$RD01\g_health";
sort hogarid;
assert _merge==3;
drop _merge;
sort hogarid;
save "$RD01\g_total01" , replace;

/***********************************************************************************************/	
/* COMPUTATION OF RENTAL EQUIVALENT OF DURABLE GOODS (SECTION 8.C)				     */
/***********************************************************************************************/

/* The method used to calculate the value of consumer durables consumption is the one         */
/* recommend by Deaton and Zaidi (2002, p. 36) when only date on current value and age of the */
/* good is available. 												    */


/* expected life-span of each product is twice its average age */
use "$RD01\equipos01";
collapse (mean) expected_lifespan=s8cp03 , by(s8cp00);
replace expected_lifespan = 2* expected_lifespan;			
sort s8cp00; 
save "$RD01\expected_lifespan" , replace;

use "$RD01\equipos01";
sort s8cp00;
merge s8cp00 using "$RD01\expected_lifespan";
assert _merge==3;

drop if s8cp01==2;
gen expected_years_remaining = expected_lifespan-s8cp03; 
replace expected_years_remaining = 1 if expected_years_remaining < 1 & expected_lifespan < 10;
replace expected_years_remaining = 2 if expected_years_remaining < 1 & expected_lifespan >= 10;
gen value_durable_cons = s8cp04/expected_years_remaining;
collapse (sum) value_durable_cons , by(hogarid);
sort hogarid;
save "$RD01\g_durables" , replace;

use "$RD01\g_total01";
merge hogarid using "$RD01\g_durables";

sort hogarid;
assert _merge~=2;
drop _merge;
recode value_durable_cons .=0;
sort hogarid;
save "$RD01\g_total01" , replace;

/***********************************************************************************************/	
/* COMPUTATION OF EXPENDITURES ON EDUCATION (SECTION 5)						     */
/***********************************************************************************************/

/* School year in Nicaragua is about 8.5 months. Therefore montly educational expenditures are */
/* multiplied by 8.5 NOT 12.																   */	

global ed_mult "8.5";

use s5p49 s5p50 s5p51 s5p69a s5p69b s5p70b s5p71b s5p72b s5p73b s5p74b s5p75b hogarid using "$RD01\poblacionlb01.dta";

/* annualize education expenditures where necessary */
replace s5p49 =s5p49 *$ed_mult;						/* school fees, snacks and meals last month */
replace s5p69a=s5p69a*s5p69b;							/* payment of monthly quotas */
replace s5p70b=s5p70b*$ed_mult;						/* monthly school fees */
replace s5p71b=s5p71b*$ed_mult;						/* monthly transportation */		

collapse (sum) s5p49 s5p50 s5p51 s5p69a s5p70b s5p71b s5p72b s5p73b s5p74b s5p75b, by (hogarid);
gen education = s5p49 + s5p50 + s5p51 + s5p69a+ s5p70b+ s5p71b + s5p72b + s5p73b + s5p74b + s5p75b;
drop  s5p49-s5p75b;

sort hogarid;
save "$RD01\g_educ" , replace;

use "$RD01\g_total01";
merge hogarid using "$RD01\g_educ";  

sort hogarid;
assert _merge==3;
drop _merge;
sort hogarid;
save "$RD01\g_total01" , replace;

/***********************************************************************************************/	
/* COMPUTATION OF EXPENDITURES ON MATERNAL HEALTH (SECTION 7)					     */
/***********************************************************************************************/

use hogarid s7p95b s7p96b s7p99b using "$RD01\poblacionlb01";
collapse (sum) s7p95b s7p96b s7p99b , by (hogarid);
gen maternal_health = s7p95b + s7p96b + s7p99b;
drop s7p95b s7p96b s7p99b;
sort hogarid;
save "$RD01\g_mhealth", replace; 

use "$RD01\g_total01";
merge hogarid using "$RD01\g_mhealth";
assert _merge==3;
drop _merge;
sort hogarid;
save "$RD01\g_total01" , replace;

/***********************************************************************************************/	
/* ORGANIZING AND CLEANING-UP OF FINAL DATASET								     */
/***********************************************************************************************/

label data "LdB 2001 : Total annual consumption";

gen total_exp 	= food + nonfood + water + garbage + electricity + lighting + cooking + telephone + rent + health + value_durable_cons + education 
		       + maternal_health;
gen total_exp_b = food_b + nonfood_b + water + garbage + electricity + lighting + cooking + telephone + rent + health + value_durable_cons + education 
		       + maternal_health;

gen food_share = food / total_exp;
gen food_share_b = food_b / total_exp_b;

label var total_exp    		"Total annual consumption expenditures in C$";
label var total_exp_b  		"Total annual consumption expenditures in C$ with outlier corrections";
label var water      		"Annual consumption : water"; 
label var garbage		      "Annual consumption : garbage"; 
label var electricity		"Annual consumption : electricity fuel"; 
label var lighting	      "Annual consumption : lighting";
label var cooking		      "Annual consumption : cooking fuel"; 
label var telephone  		"Annual consumption : telephone"; 
label var rent     		"Annual consumption : annual rent (actual or implicit)"; 
label var health     		"Annual consumption : health"; 
label var education      	"Annual consumption : education"; 
label var maternal_health     "Annual consumption : maternal health";
label var value_durable_con   "Annual consumption : imputed consumption"; 
label var food_share   		"Food consumption as a share of total consumption"; 
label var food_share_b 		"Food consumption as a share of total consumption with outlier corrections"; 

order 	hogarid i01 i02 comarca comcens i04 member non_member caloric_requirement adult_male_equivalent head_male spouse head_yrssch spouse_yrssch 
	age_0t1g age_0t1g_n age_0t1b age_0t1b_n age_1t5g age_1t5g_n age_1t5b age_1t5b_n age_5t9g age_5t9g_n age_5t9b age_5t9b_n 
	age_9t14g age_9t14g_n age_9t14b age_9t14b_n age_14t18g age_14t18g_n age_14t18b age_14t18b_n age_18t30g age_18t30g_n age_18t30b age_18t30b_n 
	age_30t60g age_30t60g_n age_30t60b age_30t60b_n age_60g age_60g_n age_60b age_60b_n 
	food cereals roots pulses veggies fruits meats dairy oils beverages meals misc other
	food_b cereals_b roots_b pulses_b veggies_b fruits_b meats_b dairy_b oils_b beverages_b meals_b misc_b other_b
	food_exp_outlier number_foods 
	nonfood nonfood_b nffood_exp_outlier number_nonfood 
	water garbage electricity lighting cooking telephone rent health value_durable_cons education maternal_health 
	total_exp total_exp_b 
	food_share food_share_b 
	error_f error_nf error_t;

format nonfood  %9.0f ;
format water    %9.0f ;
compress; 
sort hogarid;
save "$RD01\g_total01" , replace; 


/* list households with high and low levels of per capita expenditure */
g total_exp_pc_US = (total_exp/member) / $EXRATE; 
sort total_exp_pc_US;
li hogarid total_exp_pc_US if total_exp_pc_US<45, noobs nolab;
li hogarid total_exp_pc_US if total_exp_pc_US>1500 , noobs nolab; 

/***************************************************************************************************/
/* FLAG POSSIBLE OUTLIERS USING TUKEY'S METHOD									   */														 /***************************************************************************************************/
/***************************************************************************************************/

g totexp_pc=total_exp/member;
egen c_25 = pctile(totexp_pc), p(25);
egen c_75 = pctile(totexp_pc), p(75);
egen c_iqr = iqr(totexp_pc);
g lower_bd = c_25-3*c_iqr;
g upper_bd = c_75+3*c_iqr;
sum lower_bd upper_bd;
g totexp_outlier=0;
replace totexp_outlier = 1 if totexp_pc<lower_bd | totexp_pc>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;
tab totexp_outlier;

erase "$RD01\expected_lifespan.dta";
erase "$RD01\g_health.dta";
erase "$RD01\g_educ.dta";
erase "$RD01\g_durables.dta"; 
erase "$RD01\g_other.dta"; 
erase "$RD01\g_mhealth.dta"; 
erase "$RD01\tmpdigit.dta"; 

exit;
