/***********************************************************************************************/
/* Bryan S. Graham, UC - Berkeley (bgraham@econ.berkeley.edu)					     */
/* Jan 2008														     */
/***********************************************************************************************/	

/***********************************************************************************************/
/* This file is produces a household-level record of demographic characteristics, including a  */
/* measure of recommended caloric inake according to FAO/WHO data.				     */	
/***********************************************************************************************/                                                              

/* use a semicolon as the command delimiter */
#delimit ;

clear;

set matsize 800;
set memory 100m;

/***********************************************************************************************/
/* DATA FILES USED TO CONSTRUCT AGGREGATES								     */	
/* 															     */
/*     poblacionlb01.dta											     */
/*     viviendalb01.dta												     */
/***********************************************************************************************/	

/***********************************************************************************************/
/* DATA FILES CREATED BY THE PROGRAM									     */
/*   															     */	
/*     g_demo01   	   (household demographics in 2001)						     */
/***********************************************************************************************/	

/* generate file with basic household identification information */
use hogarid i01 i02 comarca i04 comcens using "$RD01\viviendalb01"; 
sort hogarid; 
save "$RD01\tmpdigit" , replace;

/* open file with household roster information */
use "$RD01\poblacionlb01", clear; 

sort hogarid;
merge hogarid using "$RD01\tmpdigit" , nokeep;
assert _merge==3;
drop _merge;

/* drop records corresponding to dead individuals */
drop if s1p06==3;

g member = 1 if miembro==1;
replace member = 0 if miembro==2;
label variable member "Number of household members";

g non_member = 1 if miembro==2;
replace non_member = 0 if miembro==1;
label variable non_member "Number of living non-members listed on household roster";

/***************************************************************************/
/* recommended caloric intake for members and non-members in the household */
/***************************************************************************/

g caloric_requirement = 650 if yearc==0 & sexo==1;
replace caloric_requirement = 600 if yearc==0 & sexo==2;
replace caloric_requirement = 950 if yearc==1 & sexo==1;
replace caloric_requirement = 850 if yearc==1 & sexo==2;
replace caloric_requirement = 1125 if yearc==2 & sexo==1;
replace caloric_requirement = 1050 if yearc==2 & sexo==2;
replace caloric_requirement = 1250 if yearc==3 & sexo==1;
replace caloric_requirement = 1150 if yearc==3 & sexo==2;
replace caloric_requirement = 1350 if yearc==4 & sexo==1;
replace caloric_requirement = 1250 if yearc==4 & sexo==2;
replace caloric_requirement = 1475 if yearc==5 & sexo==1;
replace caloric_requirement = 1325 if yearc==5 & sexo==2;
replace caloric_requirement = 1350 if yearc==6 & sexo==1;
replace caloric_requirement = 1225 if yearc==6 & sexo==2;
replace caloric_requirement = 1450 if yearc==7 & sexo==1;
replace caloric_requirement = 1325 if yearc==7 & sexo==2;
replace caloric_requirement = 1550 if yearc==8 & sexo==1;
replace caloric_requirement = 1450 if yearc==8 & sexo==2;
replace caloric_requirement = 1675 if yearc==9 & sexo==1;
replace caloric_requirement = 1575 if yearc==9 & sexo==2;
replace caloric_requirement = 1825 if yearc==10 & sexo==1;
replace caloric_requirement = 1700 if yearc==10 & sexo==2;
replace caloric_requirement = 2000 if yearc==11 & sexo==1;
replace caloric_requirement = 1825 if yearc==11 & sexo==2;
replace caloric_requirement = 2175 if yearc==12 & sexo==1;
replace caloric_requirement = 1925 if yearc==12 & sexo==2;
replace caloric_requirement = 2350 if yearc==13 & sexo==1;
replace caloric_requirement = 2025 if yearc==13 & sexo==2;
replace caloric_requirement = 2550 if yearc==14 & sexo==1;
replace caloric_requirement = 2075 if yearc==14 & sexo==2;
replace caloric_requirement = 2700 if yearc==15 & sexo==1;
replace caloric_requirement = 2125 if yearc==15 & sexo==2;
replace caloric_requirement = 2825 if yearc==16 & sexo==1;
replace caloric_requirement = 2125 if yearc==16 & sexo==2;
replace caloric_requirement = 2900 if yearc==17 & sexo==1;
replace caloric_requirement = 2125 if yearc==17 & sexo==2;
replace caloric_requirement = 2550 if yearc>=18 & year<30 & sexo==1;
replace caloric_requirement = 2025 if yearc>=18 & year<30 & sexo==2;
replace caloric_requirement = 2500 if yearc>=30 & year<60 & sexo==1;
replace caloric_requirement = 1980 if yearc>=30 & year<60 & sexo==2;
replace caloric_requirement = 2075 if yearc>=60 & sexo==1;
replace caloric_requirement = 1775 if yearc>=60 & sexo==2;
label variable caloric_requirement "UNU/WHO/FAO recommended daily caloric intake for light activity";

g adult_male_equivalent = caloric_requirement/2550;
label variable adult_male_equivalent "UNU/WHO/FAO recommended caloric intake as a fraction of that for an adult male aged 18 - 30";

/***********************************************************************/
/* education/sex of household head and spouse   			     */
/***********************************************************************/

g head_male = 0 if sexo == 2 & s2p07 == 1;
replace head_male = 1 if sexo == 1 & s2p07 == 1;
label variable head_male "Equal to 1 if household head is male";

g spouse = 1 if s2p07 == 2;
replace spouse = 0 if spouse==.;
label variable spouse "Household head has a spouse living in the home";

/* calculate years of schooling for head and spouse */
g yrssch = 0;
replace yrssch = s2p11b if s2p11a==3;
replace yrssch = 6 + s2p11b if s2p11a==4;
replace yrssch = 6 + s2p11b if s2p11a==5;
replace yrssch = 6 + 3 + s2p11b if s2p11a==6;
replace yrssch = 6 + 3 + 4 + s2p11b if s2p11a==7;
replace yrssch = 6 + 5 + s2p11b if s2p11a==8;
label variable yrssch "Years of schooling completed";

g head_yrssch = yrssch if s2p07 == 1;
label variable head_yrssch "Years of schooling completed by household head";

g spouse_yrssch = yrssch if s2p07 == 2;
label variable head_yrssch "Years of schooling completed by household head's spouse";

/***********************************************************************/
/* age and sex composition of members and non-members in the household */
/***********************************************************************/

/*************/
/* age 0 to 1*/
/*************/

g age_0t1g = 0;
replace age_0t1g = 1 if yearc==0 & sexo==2 & miembro==1;
replace caloric_requirement = 600 if yearc==0 & sexo==2 & miembro==1;
label variable age_0t1g "Number of girls <1 in household";

g age_0t1g_n = 0;
replace age_0t1g_n = 1 if yearc==0 & sexo==2 & miembro==2;
replace caloric_requirement = 600 if yearc==0 & sexo==2 & miembro==2;
label variable age_0t1g_n "Number of non-member girls <1 in household";

g age_0t1b = 0;
replace age_0t1b = 1 if yearc==0 & sexo==1 & miembro==1;
replace caloric_requirement = 650 if yearc==0 & sexo==1 & miembro==1;
label variable age_0t1b "Number of boys <1 in household";

g age_0t1b_n = 0;
replace age_0t1b_n = 1 if yearc==0 & sexo==1 & miembro==2;
replace caloric_requirement = 650 if yearc==0 & sexo==1 & miembro==2;
label variable age_0t1b_n "Number of non-member boys <1 in household";

/*************/
/* age 1 to 5*/
/*************/

g age_1t5g = 0;
replace age_1t5g = 1 if yearc>=1 & yearc<5 & sexo==2 & miembro==1;
label variable age_1t5g "Number of girls >=1 & <5 in household";

g age_1t5g_n = 0;
replace age_1t5g_n = 1 if yearc>=1 & yearc<5 & sexo==2 & miembro==2;
label variable age_1t5g_n "Number of non-member girls >=1 & <5 in household";

g age_1t5b = 0;
replace age_1t5b = 1 if yearc>=1 & yearc<5 & sexo==1 & miembro==1;
label variable age_1t5b "Number of boys >=1 & <5 in household";

g age_1t5b_n = 0;
replace age_1t5b_n = 1 if yearc>=1 & yearc<5 & sexo==1 & miembro==2;
label variable age_1t5b_n "Number of non-member boys >=1 & <5 in household";

/*************/
/* age 5 to 9*/
/*************/

g age_5t9g = 0;
replace age_5t9g = 1 if yearc>=5 & yearc<9 & sexo==2 & miembro==1;
label variable age_5t9g "Number of girls >=5 & <9 in household";

g age_5t9g_n = 0;
replace age_5t9g_n = 1 if yearc>=5 & yearc<9 & sexo==2 & miembro==2;
label variable age_5t9g_n "Number of non-member girls >=5 & <9 in household";

g age_5t9b = 0;
replace age_5t9b = 1 if yearc>=5 & yearc<9 & sexo==1 & miembro==1;
label variable age_5t9b "Number of boys >=5 & <9 in household";

g age_5t9b_n = 0;
replace age_5t9b_n = 1 if yearc>=5 & yearc<9 & sexo==1 & miembro==2;
label variable age_5t9b_n "Number of non-member boys >=5 & <9 in household";

/**************/
/* age 9 to 14*/
/**************/

g age_9t14g = 0;
replace age_9t14g = 1 if yearc>=9 & yearc<14 & sexo==2 & miembro==1;
label variable age_9t14g "Number of girls >=9 & <14 in household";

g age_9t14g_n = 0;
replace age_9t14g_n = 1 if yearc>=9 & yearc<14 & sexo==2 & miembro==2;
label variable age_9t14g_n "Number of non-member girls >=9 & <14 in household";

g age_9t14b = 0;
replace age_9t14b = 1 if yearc>=9 & yearc<14 & sexo==1 & miembro==1;
label variable age_9t14b "Number of boys >=9 & <14 in household";

g age_9t14b_n = 0;
replace age_9t14b_n = 1 if yearc>=9 & yearc<14 & sexo==1 & miembro==2;
label variable age_9t14b_n "Number of non-member boys >=9 & <14 in household";

/***************/
/* age 14 to 18*/
/***************/

g age_14t18g = 0;
replace age_14t18g = 1 if yearc>=14 & yearc<18 & sexo==2 & miembro==1;
label variable age_14t18g "Number of girls >=14 & <18 in household";

g age_14t18g_n = 0;
replace age_14t18g_n = 1 if yearc>=14 & yearc<18 & sexo==2 & miembro==2;
label variable age_14t18g_n "Number of non-member girls >=14 & <18 in household";

g age_14t18b = 0;
replace age_14t18b = 1 if yearc>=14 & yearc<18 & sexo==1 & miembro==1;
label variable age_14t18b "Number of boys >=14 & <18 in household";

g age_14t18b_n = 0;
replace age_14t18b_n = 1 if yearc>=14 & yearc<18 & sexo==1 & miembro==2;
label variable age_14t18b_n "Number of non-member boys >=14 & <18 in household";

/***************/
/* age 18 to 30*/
/***************/

g age_18t30g = 0;
replace age_18t30g = 1 if yearc>=18 & yearc<30 & sexo==2 & miembro==1;
label variable age_18t30g "Number of girls >=18 & <30 in household";

g age_18t30g_n = 0;
replace age_18t30g_n = 1 if yearc>=18 & yearc<30 & sexo==2 & miembro==2;
label variable age_18t30g_n "Number of non-member girls >=18 & <30 in household";

g age_18t30b = 0;
replace age_18t30b = 1 if yearc>=18 & yearc<30 & sexo==1 & miembro==1;
label variable age_18t30b "Number of boys >=18 & <30 in household";

g age_18t30b_n = 0;
replace age_18t30b_n = 1 if yearc>=18 & yearc<30 & sexo==1 & miembro==2;
label variable age_18t30b_n "Number of non-member boys >=18 & <30 in household";

/***************/
/* age 30 to 60*/
/***************/

g age_30t60g = 0;
replace age_30t60g = 1 if yearc>=30 & yearc<60 & sexo==2 & miembro==1;
label variable age_30t60g "Number of girls >=30 & <60 in household";

g age_30t60g_n = 0;
replace age_30t60g_n = 1 if yearc>=30 & yearc<60 & sexo==2 & miembro==2;
label variable age_30t60g_n "Number of non-member girls >=30 & <60 in household";

g age_30t60b = 0;
replace age_30t60b = 1 if yearc>=30 & yearc<60 & sexo==1 & miembro==1;
label variable age_30t60b "Number of boys >=30 & <60 in household";

g age_30t60b_n = 0;
replace age_30t60b_n = 1 if yearc>=30 & yearc<60 & sexo==1 & miembro==2;
label variable age_30t60b_n "Number of non-member boys >=30 & <60 in household";

/**************/
/* age 60 plus*/
/**************/

g age_60g = 0;
replace age_60g = 1 if yearc>=60 & sexo==2 & miembro==1;
label variable age_60g "Number of girls >=60 in household";

g age_60g_n = 0;
replace age_60g_n = 1 if yearc>=60 & sexo==2 & miembro==2;
label variable age_60g_n "Number of non-member girls >=60 in household";

g age_60b = 0;
replace age_60b = 1 if yearc>=60 & sexo==1 & miembro==1;
label variable age_60b "Number of boys >=60 in household";

g age_60b_n = 0;
replace age_60b_n = 1 if yearc>=60 & sexo==1 & miembro==2;
label variable age_60b_n "Number of non-member boys >=60 in household";

/**********************************************************************************/
/* collapse data down to household-level and save results                         */
/**********************************************************************************/

collapse (sum)	caloric_requirement member non_member adult_male_equivalent head_male spouse head_yrssch spouse_yrssch 
		age_0t1g age_0t1g_n age_0t1b age_0t1b_n age_1t5g age_1t5g_n age_1t5b age_1t5b_n age_5t9g age_5t9g_n age_5t9b age_5t9b_n 
		age_9t14g age_9t14g_n age_9t14b age_9t14b_n age_14t18g age_14t18g_n age_14t18b age_14t18b_n 
		age_18t30g age_18t30g_n age_18t30b age_18t30b_n age_30t60g age_30t60g_n age_30t60b age_30t60b_n age_60g age_60g_n age_60b age_60b_n, 
		by(comcens i01 i02 comarca i04 hogarid);

sort hogarid;
save "$RD01\g_demo01" , replace;
erase "$RD01\tmpdigit.dta";

