/******************************************************************************************************/
/* CALCULATE IRON AVAILABILITY BY HOUSEHOLD IN 2001									*/
/******************************************************************************************************/

/* use a semicolon as the command delimiter */
#delimit ;

clear;

set matsize 800;
set memory 100m;

/******************************************************************************************************/
/* Input iron mg per 100g as conversion factors for foods derived from "Tabla de 		  		*/
/* Composición de Alimentos de Centroamerica" accessed online at                                     */
/* (http://www.tabladealimentos.net/tca/TablaAlimentos/inicio.html). The data appendix provides full  */
/* details and sources.                                                                               */
/******************************************************************************************************/

use "$RD01\g_grams01", clear;
save "$RD01\g_iron01", replace;

use "$BASE\calorie_content", clear;
g food = real(code_rps);
collapse iron_mg per_edible, by(food);
sort food;
save "$BASE\temp", replace;

use "$RD01\g_iron01", clear;
sort food;
merge food using "$BASE\temp";
tab _m;
bys _m: tab food, missing;
drop if _m==2;
drop _m;
erase "$BASE\temp.dta";

log using "$RD01\iron01_log", replace;
log on;

/* check to see which foods have iron information */
tab error_unit;
sum iron_mg if error_unit==0;
sum iron_mg if error_unit==1;
tab food if iron_mg==. & error_unit==0;
tab food if iron_mg==. & error_unit==1;
log off;

/**********************************************************************************/
/* Estimate average daily iron availability by food using two methods   	    */
/* METHOD 1: using household unit data to convert directly into iron              */
/* METHOD 2: using household expenditure data and estimated prices                */
/**********************************************************************************/

gen yearly = frequency;
tab yearly, missing;
recode yearly  1=365  2=52  3=24  4=12  5=4  6=2  7=1;
assert yearly ~=.;
label var yearly "Annualization of frequency variable";

/* multiple grams available by percent edible */
replace grams = grams*per_edible;

/******************************************************************/
/* METHOD 1: calculate total daily iron available by food     	*/
/******************************************************************/

g iron = iron_mg*grams if error_unit==0;
replace iron = 0 if food == 53;					/* assume purified water has zero iron */
replace iron = 0 if food == 57;					/* assume cigarettes have zero iron */
label variable iron "daily iron (mg) available (from unit data)";

/* daily iron available by food and household */
replace iron = (iron*yearly)/365 if error_unit==0;

/* create a label for records corresponding to foods with absolutely no iron data */
bys food: egen iron_missing = sum((iron~=.));
replace iron_missing = -1 if iron_missing>0;
replace iron_missing = 1 if iron_missing==0;
replace iron_missing = 0 if iron_missing==-1;

/* save data in current form */
sort hogarid food constype;
save "$RD01\g_iron01", replace;

sort hogarid;
merge hogarid using "$RD01\g_demo01";
assert _merge==3;
drop _m;

/**********************************************************************************/
/* collapse data down to the household-by-food item level				    */
/**********************************************************************************/

collapse (sum) iron cons_yr error_unit iron_missing, by(i01 i02 comcens hogarid food);	

/* NOTE: (sum) collapse has the peculiar feature that if all units of missing in the 		 */
/*       group being summed over it will return a zero value and not a missing value 		 */
/*	   the iron_missing variable keeps track of such cases so that they can be 		 	 */
/*       properly set back equal to zero.										 */

/**********************************************************************************/
/* calculate unit level prices using median comarca iron per gram among	          */
/* records with valid units, or muncipio or survey means (if needed)		    */
/**********************************************************************************/

g daily_exp = cons_yr/365;	
g t = iron/daily_exp if error_unit==0 & iron_missing==0;

bys comcens food: egen u1 = median(t);		/* median food-specific price in a comarca */
bys comcens food: egen c1 = count(t);		/* number of unit value observations in comarca */
bys i02 food: egen u2 = median(t);			/* median food-specific price in a municipio */	
bys i02 food: egen c2 = count(t);			/* number of unit value observations in municipio */
bys food: egen u3 = median(t);			/* median food-specific price in the survey */	
bys food: egen c3 = count(t);				/* number of unit value observations in survey */

g iron_per_cordoba = u1 if c1>=10;
g str50 price_source = "comarca median" if c1>=10;
replace iron_per_cordoba = u2 if iron_per_cordoba ==. & c2>=10;
replace price_source = "municipio median" if c1<10 & c2>=10;
replace iron_per_cordoba = u3 if iron_per_cordoba ==.;
replace price_source = "survey median" if c1<10 & c2<10;
drop t u1 u2 u3 c1 c2 c3;
label variable iron_per_cordoba "Median iron (g) per cordoba within comarca-food cells";

log on;
/* distribution of price data by source */
tab price_source;
log off;

collapse (mean) iron_per_cordoba, by(comcens food);
sort comcens food;
save "$RD01\g_iron_prices01", replace; 

/* NOTE: since iron data is only available (prior to imputation) for observations with valid units 	 */
/*       this method should give sensible local price estimates (in principle)      		       */

/**********************************************************************************************/
/* impute iron for observations with invalid units using the iron price data	          	    */
/**********************************************************************************************/

use "$RD01\g_iron01", clear;
sort hogarid;
merge hogarid using "$RD01\g_demo01";
assert _merge==3;
drop _m;

sort comcens food;
merge comcens food using "$RD01\g_iron_prices01", uniqusing;
assert _merge==3;
drop _merge;

log on;
/* list foods without/with price data */
tab food if iron_per_cordoba == .;
tab food if iron_per_cordoba ~= .;
log off;

sort hogarid;
merge hogarid using "$RD01\g_demo01";
assert _merge==3;
drop _m;

g str80 source_of_iron_measure = "valid unit data" if error_unit==0 & iron~=.;
replace iron = iron_per_cordoba*value*(yearly/365) if error_unit==1 & iron==.;
replace source_of_iron_measure = "imputed using value data" if error_unit==1 & iron~=.;

log on;
/* summarize results of iron imputation procedure */
tab source_of_iron_measure, missing;
tab food if iron_per_cordoba==.;
log off;

/* save data in current form */
sort hogarid food constype;
save "$RD01\g_iron01", replace;

/*****************************************************************************************************************/
/* collapse data down to househould-by-food level and check for outliers in iron estimates based on unit data    */
/*****************************************************************************************************************/

collapse (sum) iron iron_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace iron = . if iron_missing>0;

sort hogarid; 
merge hogarid using "$RD01\g_demo01", nokeep;
assert _m == 3;
drop _m; 

/* iron per adult equivalent male by food */
g iron_ame=iron/adult_male_equivalent if iron_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(iron_ame), p(25);
bys food: egen c_75 = pctile(iron_ame), p(75);
bys food: egen c_iqr = iqr(iron_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g iron_outlier=0;
replace iron_outlier = 1 if iron_ame<lower_bd | iron_ame>upper_bd;
replace iron_outlier = . if iron_missing>0;
label variable iron_outlier "Apparent outlier in units-based iron estimate";
drop iron_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab iron_outlier error_unit, missing;
log off;

/* for outlier households replace iron available with comarca, municipio or survey median */
/* food-specific iron available adjusted for household composition */

g iron_ame = iron/adult_male_equivalent if iron_outlier==0;
bys comcens food: egen u1 = median(iron_ame);
bys comcens food: egen c1 = count(iron_ame);
bys i02 food: egen u2 = median(iron_ame);
bys i02 food: egen c2 = count(iron_ame);
bys food: egen u3 = median(iron_ame);
bys food: egen c3 = count(iron_ame);

g iron_b	   = iron if iron_outlier==0;
g str80 source_of_iron_measure_b = "valid unit/value data" if iron_outlier==0;
replace iron_b = u1*adult_male_equivalent if iron_outlier==1 & c1>=10;
replace source_of_iron_measure_b = "adult male equivalent comarca median" if iron_outlier==0 & c1>=10;
replace iron_b = u2*adult_male_equivalent if iron_outlier==1 & iron_b==. & c1<10 & c2>=10;
replace source_of_iron_measure_b = "adult male equivalent municipio median" if iron_outlier==1 & iron_b==. & c1<10 & c2>=10;
replace iron_b = u3*adult_male_equivalent if iron_outlier==1 & iron_b==.;
replace source_of_iron_measure_b = "adult male equivalent survey median" if iron_outlier==1 & iron_b==.;
label variable iron_b "Outlier corrected daily iron available (from unit data)";
label variable source_of_iron_measure_b "Source of outlier corrected iron measure";
drop u1 u2 u3 c1 c2 c3 iron_ame;

/*****************************************************************/
/* Take outlier-corrected food specific iron totals and 	     */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid iron_outlier source_of_iron_measure_b food constype iron_b;
sort hogarid food constype; 
save "$RD01\toutlier", replace;

use "$RD01\g_iron01", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD01\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD01\toutlier.dta";

/**********************************************************************************************/
/* METHOD 2: estimate iron availability using value data				                */
/**********************************************************************************************/

g iron_c = iron_per_cordoba*(cons_yr/365);

/* save data in current form */
sort hogarid food constype;
save "$RD01\g_iron01", replace;

/*****************************************************************************************************************/
/* collapse data down to household-by-food level and check for outliers in iron estimates based on unit data     */
/*****************************************************************************************************************/

collapse (sum) iron_c iron_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace iron_c = . if iron_missing>0;

sort hogarid; 
merge hogarid using "$RD01\g_demo01", nokeep;
assert _m == 3;
drop _m; 

/* iron per adult equivalent male by food */
g iron_ame=iron_c/adult_male_equivalent if iron_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(iron_ame), p(25);
bys food: egen c_75 = pctile(iron_ame), p(75);
bys food: egen c_iqr = iqr(iron_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g iron_outlier_c=0;
replace iron_outlier_c = 1 if iron_ame<lower_bd | iron_ame>upper_bd;
replace iron_outlier_c = . if iron_missing>0;
label variable iron_outlier_c "Apparent outlier in values and price-based iron estimate";
drop iron_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab iron_outlier_c error_unit, missing;
log off;

/* for outlier households replace iron available with comarca, municipio or survey median */
/* food-specific iron available adjusted for household composition */

g iron_ame = iron_c/adult_male_equivalent if iron_outlier_c==0;
bys comcens food: egen u1 = median(iron_ame);
bys comcens food: egen c1 = count(iron_ame);
bys i02 food: egen u2 = median(iron_ame);
bys i02 food: egen c2 = count(iron_ame);
bys food: egen u3 = median(iron_ame);
bys food: egen c3 = count(iron_ame);

g iron_d	   = iron_c if iron_outlier_c==0;
g str80 source_of_iron_measure_d = "valid price/value data" if iron_outlier_c==0;
replace iron_d = u1*adult_male_equivalent if iron_outlier_c==1 & c1>=10;
replace source_of_iron_measure_d = "adult male equivalent comarca median" if iron_outlier_c==0 & c1>=10;
replace iron_d = u2*adult_male_equivalent if iron_outlier_c==1 & iron_d==. & c1<10 & c2>=10;
replace source_of_iron_measure_d = "adult male equivalent municipio median" if iron_outlier_c==1 & iron_d==. & c1<10 & c2>=10;
replace iron_d = u3*adult_male_equivalent if iron_outlier_c==1 & iron_d==.;
replace source_of_iron_measure_d = "adult male equivalent survey median" if iron_outlier_c==1 & iron_d==.;
label variable iron_d "Outlier corrected daily iron (g) available (from price/value data)";
label variable source_of_iron_measure_d "Source of outlier corrected iron measure";
drop u1 u2 u3 c1 c2 c3 iron_ame;

/*****************************************************************/
/* Take outlier-corrected food specific iron totals and          */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid iron_outlier_c source_of_iron_measure_d food constype iron_d;
sort hogarid food constype; 
save "$RD01\toutlier", replace;

use "$RD01\g_iron01", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD01\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD01\toutlier.dta";

keep 		hogarid comcens i01 i02 comarca i04 food constype iron 
		iron_b iron_c iron_d 
		source_of_iron_measure source_of_iron_measure_b source_of_iron_measure_d iron_outlier iron_outlier_c
		cons_yr cons_yr_b;

/* save data in current form */
sort hogarid food constype;
save "$RD01\g_iron01", replace;

/***********************************************************************************/
/* compute household iron per capita availability less meals out			     */
/***********************************************************************************/

/*******************************************************************/
/* use second iron estimate (unit data with outlier correction) 	 */
/* use outlier corrected expenditure data as well                  */
/*******************************************************************/

keep hogarid food iron_b cons_yr_b;
rename iron_b iron;
rename cons_yr_b cons_yr;

g meals = 1 if inlist(food,58,99);
replace meals = 0 if meals==.;
g meal_exp = meals*(cons_yr/365);

g other = 1 if inlist(food,61,62);
replace other = 0 if other==.;
g other_exp = other*(cons_yr/365);

g food_exp = (cons_yr/365);

/* sum annual food expenditures over each category for each household */
collapse (sum)  iron meal_exp other_exp food_exp, by(hogarid);

sort hogarid;

/****************************************************************/
/* impute iron from prepared food, other food and meals out     */
/****************************************************************/

g iron_per_cordoba = iron/(food_exp-meal_exp-other_exp);						/* average number of iron g per C$ */
g meal_iron=(2/3)*iron_per_cordoba*meal_exp if iron_per_cordoba>0;				/* imputed meal food iron (150% mark-up)*/ 
g other_iron=iron_per_cordoba*other_exp if iron_per_cordoba>0;					/* imputed other food iron (no mark-up)*/ 

/* save household-level iron consumption totals */
merge hogarid using "$RD01\g_demo01";
assert _merge==3;
drop _m;

/************************************************************************************/
/* some households' food consumption may be entirely in the other & meals categories*/
/* in which case a iron_per_cordoba measure does not exist for them             	*/
/* use comarca medians to impute iron availability for them.                     	*/
/************************************************************************************/

bys comcens: egen t = median(iron_per_cordoba);
replace meal_iron=(2/3)*t*meal_exp if iron_per_cordoba==0 | iron_per_cordoba==.;		/* imputed meal food iron (150% mark-up)*/ 
replace other_iron=t*other_exp if iron_per_cordoba==0 | iron_per_cordoba==.;			/* imputed other food iron (no mark-up)*/ 

replace iron = iron+meal_iron+other_iron;
drop t iron_per_cordoba;

/*****************************************************************/
/* calculate cordobas per iron for each household                */
/*****************************************************************/

g iron_price = 100*food_exp/iron;
label variable iron_price "C$ per 100mg of iron";


keep hogarid iron iron_price;
			
sort hogarid;
merge hogarid using "$RD01\g_total01";
assert _merge==3;
drop _m;

g iron_ame = iron/adult_male_equivalent;
centile iron_ame, centile(0,10,25,50,75,90,100);
sort hogarid;

/* check for outliers in iron aggregate */
egen c_25 = pctile(iron_ame), p(25);
egen c_75 = pctile(iron_ame), p(75);
egen c_iqr = iqr(iron_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;
g iron_outlier=0;
replace iron_outlier = 1 if iron_ame<lower_bd | iron_ame>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* apparent outliers in the aggregate iron availability measure */
tab iron_outlier;
centile iron_ame if iron_outlier==0, centile(0,10,25,50,75,90,100);
log off;

/* calculate per capita iron availability by expenditure quartile */
g total_exp_ame = total_exp/adult_male_equivalent;
egen c_25 = pctile(total_exp_ame), p(25);
egen c_50 = pctile(total_exp_ame), p(50);
egen c_75 = pctile(total_exp_ame), p(75);

sum iron_ame iron_price if total_exp_ame<=c_25;
sum iron_ame iron_price if total_exp_ame>c_25 & total_exp_ame<=c_50;
sum iron_ame iron_price if total_exp_ame>c_50 & total_exp_ame<=c_75;
sum iron_ame iron_price if total_exp_ame>c_75;

log close;

g year = 2001;
sort hogarid;
save "$RD01\g_iro_exp_clean01", replace;
