/******************************************************************************************************/
/* CALCULATE PROTEIN AVAILABILITY BY HOUSEHOLD IN 2000										          */
/******************************************************************************************************/

/* use a semicolon as the command delimiter */
#delimit ;

clear;

set matsize 800;
set memory 100m;

/******************************************************************************************************/
/* Input protein grams per 100g as conversion factors for foods derived from "Tabla de 		  		  */
/* Composición de Alimentos de Centroamerica" accessed online at                                     */
/* (http://www.tabladealimentos.net/tca/TablaAlimentos/inicio.html). The data appendix provides full  */
/* details and sources.                                                                               */
/******************************************************************************************************/

use "$RD00\g_grams00", clear;
save "$RD00\g_protein00", replace;

use "$BASE\calorie_content", clear;
g food = real(code_rps);
collapse protein_g per_edible, by(food);
sort food;
save "$BASE\temp", replace;

use "$RD00\g_protein00", clear;
sort food;
merge food using "$BASE\temp";
tab _m;
bys _m: tab food, missing;
drop if _m==2;
drop _m;
erase "$BASE\temp.dta";

log using "$RD00\protein00_log", replace;
log on;

/* check to see which foods have protein information */
tab error_unit;
sum protein_g if error_unit==0;
sum protein_g if error_unit==1;
tab food if protein_g==. & error_unit==0;
tab food if protein_g==. & error_unit==1;
log off;

/**********************************************************************************/
/* Estimate average daily protein availability by food using two methods          */
/* METHOD 1: using household unit data to convert directly into protein           */
/* METHOD 2: using household expenditure data and estimated prices                */
/**********************************************************************************/

gen yearly = frequency;
tab yearly, missing;
recode yearly  1=365  2=52  3=24  4=12  5=4  6=2  7=1;
assert yearly ~=.;
label var yearly "Annualization of frequency variable";

/* multiple grams available by percent edible */
replace grams = grams*per_edible;

/******************************************************************/
/* METHOD 1: calculate total daily protein available by food      */
/******************************************************************/

g protein = protein_g*grams if error_unit==0;
replace protein = 0 if food == 53;					/* assume purified water has zero protein */
replace protein = 0 if food == 57;					/* assume cigarettes have zero protein */
label variable protein "daily protein (g) available (from unit data)";

/* daily protein available by food and household */
replace protein = (protein*yearly)/365 if error_unit==0;

/* create a label for records corresponding to foods with absolutely no protein data */
bys food: egen protein_missing = sum((protein~=.));
replace protein_missing = -1 if protein_missing>0;
replace protein_missing = 1 if protein_missing==0;
replace protein_missing = 0 if protein_missing==-1;

/* save data in current form */
sort hogarid food constype;
save "$RD00/g_protein00", replace;

sort hogarid;
merge hogarid using "$RD00/g_demo00";
assert _merge==3;
drop _m;

/**********************************************************************************/
/* collapse data down to the household-by-food item level		 	              */
/**********************************************************************************/

collapse (sum) protein cons_yr error_unit protein_missing, by(i01 i02 comcens hogarid food);	

/* NOTE: (sum) collapse has the peculiar feature that if all units of missing in the 		 */
/*       group being summed over it will return a zero value and not a missing value 		 */
/*	   the protein_missing variable keeps track of such cases so that they can be   		 */
/*       properly set back equal to zero.										             */

/**********************************************************************************/
/* calculate unit level prices using median comarca protein per gram among 	      */
/* records with valid units, or muncipio or survey means (if needed)		      */
/**********************************************************************************/

g daily_exp = cons_yr/365;	
g t = protein/daily_exp if error_unit==0 & protein_missing==0;

bys comcens food: egen u1 = median(t);		/* median food-specific price in a comarca */
bys comcens food: egen c1 = count(t);		/* number of unit value observations in comarca */
bys i02 food: egen u2 = median(t);			/* median food-specific price in a municipio */	
bys i02 food: egen c2 = count(t);			/* number of unit value observations in municipio */
bys food: egen u3 = median(t);				/* median food-specific price in the survey */	
bys food: egen c3 = count(t);				/* number of unit value observations in survey */

g protein_per_cordoba = u1 if c1>=10;
g str50 price_source = "comarca median" if c1>=10;
replace protein_per_cordoba = u2 if protein_per_cordoba ==. & c2>=10;
replace price_source = "municipio median" if c1<10 & c2>=10;
replace protein_per_cordoba = u3 if protein_per_cordoba ==.;
replace price_source = "survey median" if c1<10 & c2<10;
drop t u1 u2 u3 c1 c2 c3;
label variable protein_per_cordoba "Median protein (g) per cordoba within comarca-food cells";

log on;
/* distribution of price data by source */
tab price_source;
log off;

collapse (mean) protein_per_cordoba, by(comcens food);
sort comcens food;
save "$RD00\g_protein_prices00", replace; 

/* NOTE: since protein data is only available (prior to imputation) for observations with valid units */
/*       this method should give sensible local price estimates (in principle)      			*/

/**********************************************************************************************/
/* impute protein for observations with invalid units using the protein price data	          */
/**********************************************************************************************/

use "$RD00\g_protein00", clear;
sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

sort comcens food;
merge comcens food using "$RD00\g_protein_prices00", uniqusing;
assert _merge==3;
drop _merge;

log on;
/* list foods without/with price data */
tab food if protein_per_cordoba == .;
tab food if protein_per_cordoba ~= .;
log off;

sort hogarid;
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

g str80 source_of_protein_measure = "valid unit data" if error_unit==0 & protein~=.;
replace protein = protein_per_cordoba*value*(yearly/365) if error_unit==1 & protein==.;
replace source_of_protein_measure = "imputed using value data" if error_unit==1 & protein~=.;

log on;
/* summarize results of protein imputation procedure */
tab source_of_protein_measure, missing;
tab food if protein_per_cordoba==.;
log off;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_protein00", replace;

/*****************************************************************************************************************/
/* collapse data down to househould-by-food level and check for outliers in protein estimates based on unit data */
/*****************************************************************************************************************/

collapse (sum) protein protein_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace protein = . if protein_missing>0;

sort hogarid; 
merge hogarid using "$RD00\g_demo00", nokeep;
assert _m == 3;
drop _m; 

/* protein per adult equivalent male by food */
g protein_ame=protein/adult_male_equivalent if protein_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(protein_ame), p(25);
bys food: egen c_75 = pctile(protein_ame), p(75);
bys food: egen c_iqr = iqr(protein_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g protein_outlier=0;
replace protein_outlier = 1 if protein_ame<lower_bd | protein_ame>upper_bd;
replace protein_outlier = . if protein_missing>0;
label variable protein_outlier "Apparent outlier in units-based protein estimate";
drop protein_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab protein_outlier error_unit, missing;
log off;

/* for outlier households replace protein available with comarca, municipio or survey median */
/* food-specific protein available adjusted for household composition */

g protein_ame = protein/adult_male_equivalent if protein_outlier==0;
bys comcens food: egen u1 = median(protein_ame);
bys comcens food: egen c1 = count(protein_ame);
bys i02 food: egen u2 = median(protein_ame);
bys i02 food: egen c2 = count(protein_ame);
bys food: egen u3 = median(protein_ame);
bys food: egen c3 = count(protein_ame);

g protein_b	   = protein if protein_outlier==0;
g str80 source_of_protein_measure_b = "valid unit/value data" if protein_outlier==0;
replace protein_b = u1*adult_male_equivalent if protein_outlier==1 & c1>=10;
replace source_of_protein_measure_b = "adult male equivalent comarca median" if protein_outlier==0 & c1>=10;
replace protein_b = u2*adult_male_equivalent if protein_outlier==1 & protein_b==. & c1<10 & c2>=10;
replace source_of_protein_measure_b = "adult male equivalent municipio median" if protein_outlier==1 & protein_b==. & c1<10 & c2>=10;
replace protein_b = u3*adult_male_equivalent if protein_outlier==1 & protein_b==.;
replace source_of_protein_measure_b = "adult male equivalent survey median" if protein_outlier==1 & protein_b==.;
label variable protein_b "Outlier corrected daily protein available (from unit data)";
label variable source_of_protein_measure_b "Source of outlier corrected protein measure";
drop u1 u2 u3 c1 c2 c3 protein_ame;

/*****************************************************************/
/* Take outlier-corrected food specific protein totals and       */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid protein_outlier source_of_protein_measure_b food constype protein_b;
sort hogarid food constype; 
save "$RD00\toutlier", replace;

use "$RD00\g_protein00", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD00\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD00\toutlier.dta";

/**********************************************************************************************/
/* METHOD 2: estimate protein availability using value data				                      */
/**********************************************************************************************/

g protein_c = protein_per_cordoba*(cons_yr/365);

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_protein00", replace;

/*****************************************************************************************************************/
/* collapse data down to household-by-food level and check for outliers in protein estimates based on unit data  */
/*****************************************************************************************************************/

collapse (sum) protein_c protein_missing error_unit (max) constype, by(i01 i02 comcens hogarid food);
replace protein_c = . if protein_missing>0;

sort hogarid; 
merge hogarid using "$RD00\g_demo00", nokeep;
assert _m == 3;
drop _m; 

/* protein per adult equivalent male by food */
g protein_ame=protein_c/adult_male_equivalent if protein_missing==0;				

/* find outliers using Tukey's methods */
bys food: egen c_25 = pctile(protein_ame), p(25);
bys food: egen c_75 = pctile(protein_ame), p(75);
bys food: egen c_iqr = iqr(protein_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;

g protein_outlier_c=0;
replace protein_outlier_c = 1 if protein_ame<lower_bd | protein_ame>upper_bd;
replace protein_outlier_c = . if protein_missing>0;
label variable protein_outlier_c "Apparent outlier in values and price-based protein estimate";
drop protein_ame c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* show number of household-by-food records with valid unit data that are apparent outliers */
tab protein_outlier_c error_unit, missing;
log off;

/* for outlier households replace protein available with comarca, municipio or survey median */
/* food-specific protein available adjusted for household composition */

g protein_ame = protein_c/adult_male_equivalent if protein_outlier_c==0;
bys comcens food: egen u1 = median(protein_ame);
bys comcens food: egen c1 = count(protein_ame);
bys i02 food: egen u2 = median(protein_ame);
bys i02 food: egen c2 = count(protein_ame);
bys food: egen u3 = median(protein_ame);
bys food: egen c3 = count(protein_ame);

g protein_d	   = protein_c if protein_outlier_c==0;
g str80 source_of_protein_measure_d = "valid price/value data" if protein_outlier_c==0;
replace protein_d = u1*adult_male_equivalent if protein_outlier_c==1 & c1>=10;
replace source_of_protein_measure_d = "adult male equivalent comarca median" if protein_outlier_c==0 & c1>=10;
replace protein_d = u2*adult_male_equivalent if protein_outlier_c==1 & protein_d==. & c1<10 & c2>=10;
replace source_of_protein_measure_d = "adult male equivalent municipio median" if protein_outlier_c==1 & protein_d==. & c1<10 & c2>=10;
replace protein_d = u3*adult_male_equivalent if protein_outlier_c==1 & protein_d==.;
replace source_of_protein_measure_d = "adult male equivalent survey median" if protein_outlier_c==1 & protein_d==.;
label variable protein_d "Outlier corrected daily protein (g) available (from price/value data)";
label variable source_of_protein_measure_d "Source of outlier corrected protein measure";
drop u1 u2 u3 c1 c2 c3 protein_ame;

/*****************************************************************/
/* Take outlier-corrected food specific protein totals and       */
/* merge back with original data.                                */
/*****************************************************************/

keep hogarid protein_outlier_c source_of_protein_measure_d food constype protein_d;
sort hogarid food constype; 
save "$RD00\toutlier", replace;

use "$RD00\g_protein00", clear;
sort hogarid food constype;
merge hogarid food constype using "$RD00\toutlier", nokeep;

assert _merge ~= 2;
drop _merge;
erase "$RD00\toutlier.dta";

keep 	hogarid comcens i01 i02 comarca i04 food constype protein 
	protein_b protein_c protein_d 
	source_of_protein_measure source_of_protein_measure_b source_of_protein_measure_d protein_outlier protein_outlier_c
	cons_yr cons_yr_b;

/* save data in current form */
sort hogarid food constype;
save "$RD00\g_protein00", replace;

/***********************************************************************************/
/* compute household protein per capita availability less meals out		           */
/***********************************************************************************/

/*******************************************************************/
/* use second protein estimate (unit data with outlier correction) */
/* use outlier corrected expenditure data as well                  */
/*******************************************************************/

keep hogarid food protein_b cons_yr_b;
rename protein_b protein;
rename cons_yr_b cons_yr;

g meals = 1 if inlist(food,58,99);
replace meals = 0 if meals==.;
g meal_exp = meals*(cons_yr/365);

g other = 1 if inlist(food,61,62);
replace other = 0 if other==.;
g other_exp = other*(cons_yr/365);

g food_exp = (cons_yr/365);

/* sum annual food expenditures over each category for each household */
collapse (sum)  protein meal_exp other_exp food_exp, by(hogarid);

sort hogarid;


/****************************************************************/
/* impute protein from prepared food, other food and meals out  */
/****************************************************************/

g protein_per_cordoba = protein/(food_exp-meal_exp-other_exp);						/* average number of protein g per C$ */
g meal_protein=(2/3)*protein_per_cordoba*meal_exp if protein_per_cordoba>0;				/* imputed meal food protein (150% mark-up)*/ 
g other_protein=protein_per_cordoba*other_exp if protein_per_cordoba>0;					/* imputed other food protein (no mark-up)*/ 

/* save household-level protein consumption totals */
merge hogarid using "$RD00\g_demo00";
assert _merge==3;
drop _m;

/************************************************************************************/
/* some households' food consumption may be entirely in the other & meals categories*/
/* in which case a protein_per_cordoba measure does not exist for them             	*/
/* use comarca medians to impute protein availability for them.                     */
/************************************************************************************/

bys comcens: egen t = median(protein_per_cordoba);
replace meal_protein=(2/3)*t*meal_exp if protein_per_cordoba==0 | protein_per_cordoba==.;			/* imputed meal food protein (150% mark-up)*/ 
replace other_protein=t*other_exp if protein_per_cordoba==0 | protein_per_cordoba==.;			/* imputed other food protein (no mark-up)*/ 

replace protein = protein+meal_protein+other_protein;
drop t protein_per_cordoba;

/*****************************************************************/
/* calculate cordobas per protein for each household             */
/*****************************************************************/

g protein_price = 100*food_exp/protein;
label variable protein_price "C$ per 100g of protein";


keep hogarid protein protein_price;
			
sort hogarid;
merge hogarid using "$RD00\g_total00";
assert _merge==3;
drop _m;

g protein_ame = protein/adult_male_equivalent;
centile protein_ame, centile(0,10,25,50,75,90,100);
sort hogarid;

/* check for outliers in protein aggregate */
egen c_25 = pctile(protein_ame), p(25);
egen c_75 = pctile(protein_ame), p(75);
egen c_iqr = iqr(protein_ame);
g lower_bd = c_25-1.5*c_iqr;
g upper_bd = c_75+1.5*c_iqr;
g protein_outlier=0;
replace protein_outlier = 1 if protein_ame<lower_bd | protein_ame>upper_bd;
drop c_25 c_75 c_iqr lower_bd upper_bd;

log on;
/* apparent outliers in the aggregate protein availability measure */
tab protein_outlier;
centile protein_ame if protein_outlier==0, centile(0,10,25,50,75,90,100);
log off;

/* calculate per capita protein availability by expenditure quartile */
g total_exp_ame = total_exp/adult_male_equivalent;
egen c_25 = pctile(total_exp_ame), p(25);
egen c_50 = pctile(total_exp_ame), p(50);
egen c_75 = pctile(total_exp_ame), p(75);

sum protein_ame protein_price if total_exp_ame<=c_25;
sum protein_ame protein_price if total_exp_ame>c_25 & total_exp_ame<=c_50;
sum protein_ame protein_price if total_exp_ame>c_50 & total_exp_ame<=c_75;
sum protein_ame protein_price if total_exp_ame>c_75;

log close;

g year = 2000;
sort hogarid;
save "$RD00\g_pro_exp_clean00", replace;
